"""File operations for User authorization."""

from sqlalchemy import exists
from sqlalchemy.orm.exc import NoResultFound
from hydrus.data.exceptions import UserExists, UserNotFound
from hydrus.data.db_models import User, Token, Nonce
from hashlib import sha224
import base64
# import random
from sqlalchemy.orm.session import Session
from werkzeug.local import LocalProxy
from random import randrange
from datetime import datetime, timedelta
from uuid import uuid4


def add_user(id_: int, paraphrase: str, session: Session) -> None:
    """Add new users to the database.

    Raises:
        UserExits: If a user with `id_` already exists.

    """
    if session.query(exists().where(User.id == id_)).scalar():
        raise UserExists(id_=id_)
    else:
        new_user = User(id=id_, paraphrase=sha224(
            paraphrase.encode('utf-8')).hexdigest())
        session.add(new_user)
        session.commit()


def check_nonce(request: LocalProxy, session: Session) -> bool:
    """check validity of nonce passed by the user."""
    try:
        id_ = request.headers['X-Authentication']
        nonce = session.query(Nonce).filter(Nonce.id == id_).one()
        present = datetime.now()
        present = present - nonce.timestamp
        session.delete(nonce)
        session.commit()
        if present > timedelta(0, 0, 0, 0, 1, 0, 0):
            return False
    except BaseException:
        return False
    return True


def create_nonce(session: Session) -> str:
    """
    Create a one time use nonce valid for a short time
    for user authentication.
    """
    nonce = str(uuid4())
    time = datetime.now()
    new_nonce = Nonce(id=nonce, timestamp=time)
    session.add(new_nonce)
    session.commit()
    return nonce


def add_token(request: LocalProxy, session: Session) -> str:
    """
    Create a new token for the user or return a
    valid existing token to the user.
    """
    token = None
    id_ = int(request.authorization['username'])
    try:
        token = session.query(Token).filter(Token.user_id == id_).one()
        if not token.is_valid():
            update_token = '%030x' % randrange(16**30)
            token.id = update_token
            token.timestamp = datetime.now()
            session.commit()
    except NoResultFound:
        token = '%030x' % randrange(16**30)
        new_token = Token(user_id=id_, id=token)
        session.add(new_token)
        session.commit()
        return token
    return token.id


def check_token(request: LocalProxy, session: Session) -> bool:
    """
    check validity of the token passed by the user.
    """
    token = None
    try:
        id_ = request.headers['X-Authorization']
        token = session.query(Token).filter(Token.id == id_).one()
        if not token.is_valid():
            token.delete()
            return False
    except BaseException:
        return False
    return True


def generate_basic_digest(id_: int, paraphrase: str) -> str:
    """Create the digest to be added to the HTTP Authorization header."""
    paraphrase_digest = sha224(paraphrase.encode('utf-8')).hexdigest()
    credentials = f'{id_}:{paraphrase_digest}'
    digest = base64.b64encode(credentials.encode('utf-8')).decode('utf-8')
    return digest


def authenticate_user(id_: int, paraphrase: str, session: Session) -> bool:
    """Authenticate a user based on the ID and his paraphrase.

    Raises:
        UserNotFound: If a user with `id_` is not a valid/defined User

    """
    user = None
    try:
        user = session.query(User).filter(User.id == id_).one()
    except NoResultFound:
        raise UserNotFound(id_=id_)
    hashvalue = user.paraphrase
    generated_hash = sha224(paraphrase.encode('utf-8')).hexdigest()

    return generated_hash == hashvalue


def check_authorization(request: LocalProxy, session: Session) -> bool:
    """Check if the request object has the correct authorization."""
    auth = request.authorization
    if check_nonce(request, session):
        return authenticate_user(auth.username, auth.password, session)
    return False



"""Main route for the application"""

import logging

from sqlalchemy import create_engine

from gevent.pywsgi import WSGIServer
from sqlalchemy.orm import sessionmaker

from hydrus.app_factory import app_factory
from hydrus.conf import (
    HYDRUS_SERVER_URL, API_NAME, DB_URL, APIDOC_OBJ, PORT, DEBUG)
from hydrus.data import doc_parse
from hydrus.data.db_models import Base, create_database_tables
from hydrus.data.exceptions import UserExists
from hydrus.data.user import add_user
from hydra_python_core import doc_maker
from hydrus.utils import (
    set_session, set_doc, set_hydrus_server_url,
    set_token, set_api_name, set_authentication)
from hydrus.socketio_factory import create_socket

logger = logging.getLogger(__file__)

# TODO: loading the engine and creating the tables should be handled better
engine = create_engine(DB_URL)
session = sessionmaker(bind=engine)()

#
# Load ApiDoc with doc_maker
#
apidoc = doc_maker.create_doc(APIDOC_OBJ, HYDRUS_SERVER_URL, API_NAME)
classes = doc_parse.get_classes(apidoc)
try:
    Base.metadata.drop_all(engine)
    create_database_tables(classes)
    Base.metadata.create_all(engine)
except Exception:
    pass

AUTH = True
TOKEN = True

if AUTH:
    try:
        add_user(id_=1, paraphrase="test", session=session)
    except UserExists:
        pass

# Create a Hydrus app
app = app_factory(API_NAME)
socketio = create_socket(app, session)
#
# Nested context managers
#
# Use authentication for all requests
# Set the API Documentation
# Set HYDRUS_SERVER_URL
# Set the Database session
with set_authentication(app, AUTH), set_token(app, TOKEN), \
     set_api_name(app, API_NAME), set_doc(app, apidoc), \
     set_hydrus_server_url(app, HYDRUS_SERVER_URL), set_session(app, session):
        if __name__ == "__main__":
            # this is run only if development server is run
            # Set the name of the API
            socketio.run(app=app, debug=True, port=PORT)
        else:
            # Start the Hydrus app
            http_server = WSGIServer(('', PORT), app)
            logger.info(f'Running server at port {PORT}')
            try:
                http_server.serve_forever()
            except KeyboardInterrupt:
                pass



"""Exceptions for the crud operations."""
from typing import Dict, Tuple, Union, Any, List
from hydra_python_core.doc_writer import HydraError


class ClassNotFound(Exception):
    """Error when the RDFClass is not found."""

    def __init__(self, type_: str) -> None:
        """Constructor."""
        self.type_ = type_

    def get_HTTP(self) -> HydraError:
        """Return the HTTP response for the Exception."""
        description = f"The class {self.type_} is not a valid/defined RDFClass"
        return HydraError(code=400, title="Invalid class", desc=description)


class InstanceNotFound(Exception):
    """Error when the Instance is not found."""

    def __init__(self, type_: str, id_: Union[str, None]=None) -> None:
        """Constructor."""
        self.type_ = type_
        self.id_ = id_

    def get_HTTP(self) -> HydraError:
        """Return the HTTP response for the Exception."""
        if str(self.id_) is None:
            description = f"Instance of type {self.type_} not found"
            return HydraError(code=404, title="Instance not found", desc=description)
        else:
            description = f"Instance of type {self.type_} with ID {str(self.id_)} not found"
            return HydraError(code=404, title="Instance not found", desc=description)


class MemberInstanceNotFound(Exception):
    """Error when the Instance (Member in Collection) is not found."""

    def __init__(self, type_: str, collection_id_: Union[str, None]=None,
                 member_id_: Union[str, None]=None) -> None:
        """Constructor."""
        self.type_ = type_
        self.collection_id = collection_id_
        self.member_id = member_id_

    def get_HTTP(self) -> HydraError:
        """Return the HTTP response for the Exception."""
        if str(self.member_id) is None:
            description = f"Instance of type {self.type_} not found"
            return HydraError(code=404, title="Instance not found", desc=description)
        else:
            description = (f"Instance of type {self.type_} with"
                           f" Collection ID {str(self.collection_id)}"
                           f" and member_id {str(self.member_id)} not found")
            return HydraError(code=404, title="Instance not found", desc=description)


class PropertyNotFound(Exception):
    """Error when the Property is not found."""

    def __init__(self, type_: str) -> None:
        """Constructor."""
        self.type_ = type_

    def get_HTTP(self) -> HydraError:
        """Return the HTTP response for the Exception."""
        description = f"The property {self.type_} is not a valid/defined Property"
        return HydraError(code=400, title="Property not found", desc=description)


class InstanceExists(Exception):
    """Error when the Instance already exists."""

    def __init__(self, type_: str, id_: Union[str, None]=None) -> None:
        """Constructor."""
        self.type_ = type_
        self.id_ = id_

    def get_HTTP(self) -> HydraError:
        """Return the HTTP response for the Exception."""
        if str(self.id_) is None:
            description = f"Instance of type {self.type_} already exists"
            return HydraError(code=400, title="Instance already exists.", desc=description)
        else:
            description = f"Instance of type {self.type_} with ID {str(self.id_)} already exists"
            return HydraError(code=400, title="Instance already exists.", desc=description)


class UserExists(Exception):
    """Error when the User already exitst."""

    def __init__(self, id_: int) -> None:
        """Constructor."""
        self.id_ = id_

    def get_HTTP(self) -> HydraError:
        """Return the HTTP response for the Exception."""
        description = f"The user with ID {self.id_} already exists"
        return HydraError(code=400, title="User already exists.", desc=description)


class UserNotFound(Exception):
    """Error when the User is not found."""

    def __init__(self, id_: int) -> None:
        """Constructor."""
        self.id_ = id_

    def get_HTTP(self) -> HydraError:
        """Return the HTTP response for the Exception."""
        description = f"The User with ID {self.id_} is not a valid/defined User"
        return HydraError(code=400, title="User not found", desc=description)


class PageNotFound(Exception):
    """Error when the page is not found."""

    def __init__(self, page_id: str) -> None:
        """Constructor."""
        self.page_id = page_id

    def get_HTTP(self) -> HydraError:
        """Return the HTTP response for the Exception."""
        description = f"The page with ID {self.page_id} not found"
        return HydraError(code=400, title="Page not found", desc=description)


class InvalidSearchParameter(Exception):
    "Error when client uses invalid query parameter for searching."

    def __init__(self, param: str) -> None:
        """Constructor."""
        self.param = param

    def get_HTTP(self) -> HydraError:
        """Return the HTTP response for the Exception."""
        description = f"Query parameter [{self.param}] is invalid"
        return HydraError(code=400, title="Invalid query parameter", desc=description)


class IncompatibleParameters(Exception):
    """Error when two or more query parameters are incompatible with each other."""

    def __init__(self, params: List[str]) -> None:
        """Constructor."""
        self.params = params

    def get_HTTP(self) -> HydraError:
        """Return the HTTP response for the Exception."""
        description = "Following parameters are incompatible with each other: ["
        for i in range(len(self.params)):
            if i == len(self.params) - 1:
                description += f"{self.params[i]}]"
            else:
                description += f"{self.params[i]}, "
        return HydraError(code=400, title="Incompatible parameters.", desc=description)


class OffsetOutOfRange(Exception):
    """Error when the offset provided by client is out of range"""

    def __init__(self, offset: str) -> None:
        """Constructor."""
        self.offset = offset

    def get_HTTP(self) -> HydraError:
        """Return the HTTP response for the Exception."""
        description = f"Offset {self.offset} is out of range."
        return HydraError(code=400, title="Page not found", desc=description)


class PropertyNotGiven(Exception):
    """Error when a Property is not given."""

    def __init__(self, type_: str) -> None:
        """Constructor."""
        self.type_ = type_

    def get_HTTP(self) -> HydraError:
        """Return the HTTP response for the Exception."""
        description = f"The property {self.type_} is not given."
        return HydraError(code=400, title="Property not given", desc=description)



"""Generated API Documentation sample using doc_writer_sample.py."""
    
doc = {
    "@context": {
        "ApiDocumentation": "hydra:ApiDocumentation",
        "description": "hydra:description",
        "domain": {
            "@id": "rdfs:domain",
            "@type": "@id"
        },
        "entrypoint": {
            "@id": "hydra:entrypoint",
            "@type": "@id"
        },
        "expects": {
            "@id": "hydra:expects",
            "@type": "@id"
        },
        "expectsHeader": "hydra:expectsHeader",
        "hydra": "http://www.w3.org/ns/hydra/core#",
        "label": "rdfs:label",
        "manages": "hydra:manages",
        "method": "hydra:method",
        "object": {
            "@id": "hydra:object",
            "@type": "@id"
        },
        "possibleStatus": "hydra:possibleStatus",
        "property": {
            "@id": "hydra:property",
            "@type": "@id"
        },
        "range": {
            "@id": "rdfs:range",
            "@type": "@id"
        },
        "rdf": "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
        "rdfs": "http://www.w3.org/2000/01/rdf-schema#",
        "readable": "hydra:readable",
        "required": "hydra:required",
        "returns": {
            "@id": "hydra:returns",
            "@type": "@id"
        },
        "returnsHeader": "hydra:returnsHeader",
        "search": "hydra:search",
        "statusCode": "hydra:statusCode",
        "subClassOf": {
            "@id": "rdfs:subClassOf",
            "@type": "@id"
        },
        "subject": {
            "@id": "hydra:subject",
            "@type": "@id"
        },
        "supportedClass": "hydra:supportedClass",
        "supportedOperation": "hydra:supportedOperation",
        "supportedProperty": "hydra:supportedProperty",
        "title": "hydra:title",
        "writeable": "hydra:writeable"
    },
    "@id": "http://hydrus.com/api/vocab",
    "@type": "ApiDocumentation",
    "description": "Description for the API Documentation",
    "entrypoint": "http://hydrus.com/api",
    "possibleStatus": [],
    "supportedClass": [
        {
            "@id": "http://hydrus.com/api/vocab?resource=dummyClass",
            "@type": "hydra:Class",
            "description": "A dummyClass for demo",
            "supportedOperation": [
                {
                    "@type": "http://schema.org/UpdateAction",
                    "expects": "http://hydrus.com/api/vocab?resource=dummyClass",
                    "expectsHeader": [],
                    "method": "POST",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "dummyClass updated.",
                            "statusCode": 200,
                            "title": ""
                        }
                    ],
                    "returns": "null",
                    "returnsHeader": [
                        "Content-Type",
                        "Content-Length"
                    ],
                    "title": "UpdateClass"
                },
                {
                    "@type": "http://schema.org/DeleteAction",
                    "expects": "null",
                    "expectsHeader": [],
                    "method": "DELETE",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "dummyClass deleted.",
                            "statusCode": 200,
                            "title": ""
                        }
                    ],
                    "returns": "null",
                    "returnsHeader": [],
                    "title": "DeleteClass"
                },
                {
                    "@type": "http://schema.org/AddAction",
                    "expects": "http://hydrus.com/api/vocab?resource=dummyClass",
                    "expectsHeader": [],
                    "method": "PUT",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "dummyClass successfully added.",
                            "statusCode": 201,
                            "title": ""
                        }
                    ],
                    "returns": "null",
                    "returnsHeader": [],
                    "title": "AddClass"
                },
                {
                    "@type": "http://schema.org/FindAction",
                    "expects": "null",
                    "expectsHeader": [],
                    "method": "GET",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "dummyClass returned.",
                            "statusCode": 200,
                            "title": ""
                        }
                    ],
                    "returns": "http://hydrus.com/api/vocab?resource=dummyClass",
                    "returnsHeader": [],
                    "title": "GetClass"
                }
            ],
            "supportedProperty": [
                {
                    "@type": "SupportedProperty",
                    "property": "http://props.hydrus.com/prop1",
                    "readable": "false",
                    "required": "false",
                    "title": "Prop1",
                    "writeable": "true"
                },
                {
                    "@type": "SupportedProperty",
                    "property": "http://props.hydrus.com/prop2",
                    "readable": "false",
                    "required": "false",
                    "title": "Prop2",
                    "writeable": "true"
                }
            ],
            "title": "dummyClass"
        },
        {
            "@id": "http://hydrus.com/api/vocab?resource=extraClass",
            "@type": "hydra:Class",
            "description": "Class without any explicit methods",
            "supportedOperation": [],
            "supportedProperty": [],
            "title": "extraClass"
        },
        {
            "@id": "http://hydrus.com/api/vocab?resource=singleClass",
            "@type": "hydra:Class",
            "description": "A non collection class",
            "supportedOperation": [
                {
                    "@type": "http://schema.org/UpdateAction",
                    "expects": "http://hydrus.com/api/vocab?resource=singleClass",
                    "expectsHeader": [],
                    "method": "POST",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "singleClass changed.",
                            "statusCode": 200,
                            "title": ""
                        }
                    ],
                    "returns": "null",
                    "returnsHeader": [],
                    "title": "UpdateClass"
                },
                {
                    "@type": "http://schema.org/DeleteAction",
                    "expects": "null",
                    "expectsHeader": [],
                    "method": "DELETE",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "singleClass deleted.",
                            "statusCode": 200,
                            "title": ""
                        }
                    ],
                    "returns": "null",
                    "returnsHeader": [],
                    "title": "DeleteClass"
                },
                {
                    "@type": "http://schema.org/AddAction",
                    "expects": "http://hydrus.com/api/vocab?resource=singleClass",
                    "expectsHeader": [],
                    "method": "PUT",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "singleClass successfully added.",
                            "statusCode": 201,
                            "title": ""
                        }
                    ],
                    "returns": "null",
                    "returnsHeader": [],
                    "title": "AddClass"
                },
                {
                    "@type": "http://schema.org/FindAction",
                    "expects": "null",
                    "expectsHeader": [],
                    "method": "GET",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "singleClass returned.",
                            "statusCode": 200,
                            "title": ""
                        }
                    ],
                    "returns": "http://hydrus.com/api/vocab?resource=singleClass",
                    "returnsHeader": [],
                    "title": "GetClass"
                }
            ],
            "supportedProperty": [
                {
                    "@type": "SupportedProperty",
                    "property": "http://props.hydrus.com/prop1",
                    "readable": "false",
                    "required": "false",
                    "title": "Prop1",
                    "writeable": "true"
                },
                {
                    "@type": "SupportedProperty",
                    "property": "http://props.hydrus.com/prop2",
                    "readable": "false",
                    "required": "false",
                    "title": "Prop2",
                    "writeable": "true"
                },
                {
                    "@type": "SupportedProperty",
                    "property": {
                        "@id": "http://hydrus.com/api/vocab?resource=singleClass/dummyProp",
                        "@type": "hydra:Link",
                        "description": "",
                        "domain": "http://hydrus.com/api/vocab?resource=singleClass",
                        "range": "http://hydrus.com/api/vocab?resource=dummyClass",
                        "supportedOperation": [],
                        "title": "dummyProp"
                    },
                    "readable": "false",
                    "required": "false",
                    "title": "dummyProp",
                    "writeable": "true"
                },
                {
                    "@type": "SupportedProperty",
                    "property": "http://hydrus.com/api/vocab?resource=anotherSingleClass",
                    "readable": "false",
                    "required": "false",
                    "title": "singleClassProp",
                    "writeable": "true"
                }
            ],
            "title": "singleClass"
        },
        {
            "@id": "http://hydrus.com/api/vocab?resource=anotherSingleClass",
            "@type": "hydra:Class",
            "description": "An another non collection class",
            "supportedOperation": [
                {
                    "@type": "http://schema.org/FindAction",
                    "expects": "null",
                    "expectsHeader": [],
                    "method": "GET",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "anotherSingleClass returned.",
                            "statusCode": 200,
                            "title": ""
                        }
                    ],
                    "returns": "http://hydrus.com/api/vocab?resource=anotherSingleClass",
                    "returnsHeader": [],
                    "title": "GetClass"
                }
            ],
            "supportedProperty": [
                {
                    "@type": "SupportedProperty",
                    "property": "http://props.hydrus.com/prop1",
                    "readable": "false",
                    "required": "false",
                    "title": "Prop1",
                    "writeable": "true"
                }
            ],
            "title": "anotherSingleClass"
        },
        {
            "@id": "http://www.w3.org/ns/hydra/core#Resource",
            "@type": "hydra:Class",
            "description": "null",
            "supportedOperation": [],
            "supportedProperty": [],
            "title": "Resource"
        },
        {
            "@id": "http://www.w3.org/ns/hydra/core#Collection",
            "@type": "hydra:Class",
            "description": "null",
            "supportedOperation": [],
            "supportedProperty": [
                {
                    "@type": "SupportedProperty",
                    "property": "http://www.w3.org/ns/hydra/core#member",
                    "readable": "false",
                    "required": "null",
                    "title": "members",
                    "writeable": "false"
                }
            ],
            "title": "Collection"
        },
        {
            "@id": "http://hydrus.com/api/vocab?resource=Extraclasses",
            "@type": "Collection",
            "description": "This collection comprises of instances of ExtraClass",
            "manages": {
                "object": "http://hydrus.com/api/vocab?resource=extraClass",
                "property": "rdf:type"
            },
            "subClassOf": "http://www.w3.org/ns/hydra/core#Collection",
            "supportedOperation": [
                {
                    "@id": "_:Extraclasses_retrieve",
                    "@type": "http://schema.org/FindAction",
                    "description": "Retrieves all the members of Extraclasses",
                    "expects": "null",
                    "expectsHeader": [],
                    "method": "GET",
                    "possibleStatus": [],
                    "returns": "http://hydrus.com/api/vocab?resource=extraClass",
                    "returnsHeader": []
                },
                {
                    "@id": "_:Extraclasses_create",
                    "@type": "http://schema.org/AddAction",
                    "description": "Create new member in Extraclasses",
                    "expects": "http://hydrus.com/api/vocab?resource=extraClass",
                    "expectsHeader": [],
                    "method": "PUT",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "A new member in Extraclasses created",
                            "statusCode": 201,
                            "title": ""
                        }
                    ],
                    "returns": "http://hydrus.com/api/vocab?resource=extraClass",
                    "returnsHeader": []
                },
                {
                    "@id": "_:Extraclasses_update",
                    "@type": "http://schema.org/UpdateAction",
                    "description": "Update member of  Extraclasses ",
                    "expects": "http://hydrus.com/api/vocab?resource=extraClass",
                    "expectsHeader": [],
                    "method": "POST",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "If the entity was updatedfrom Extraclasses.",
                            "statusCode": 200,
                            "title": ""
                        }
                    ],
                    "returns": "http://hydrus.com/api/vocab?resource=extraClass",
                    "returnsHeader": []
                },
                {
                    "@id": "_:Extraclasses_delete",
                    "@type": "http://schema.org/DeleteAction",
                    "description": "Delete member of Extraclasses ",
                    "expects": "http://hydrus.com/api/vocab?resource=extraClass",
                    "expectsHeader": [],
                    "method": "DELETE",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "If entity was deletedsuccessfully from Extraclasses.",
                            "statusCode": 200,
                            "title": ""
                        }
                    ],
                    "returns": "http://hydrus.com/api/vocab?resource=extraClass",
                    "returnsHeader": []
                }
            ],
            "supportedProperty": [
                {
                    "@type": "SupportedProperty",
                    "description": "The members of Extraclasses",
                    "property": "http://www.w3.org/ns/hydra/core#member",
                    "readable": "true",
                    "required": "false",
                    "title": "members",
                    "writeable": "true"
                }
            ],
            "title": "Extraclasses"
        },
        {
            "@id": "http://hydrus.com/api/vocab?resource=dummyclasses",
            "@type": "Collection",
            "description": "This collection comprises of instances of dummyClass",
            "manages": {
                "object": "http://hydrus.com/api/vocab?resource=dummyClass",
                "property": "rdf:type"
            },
            "subClassOf": "http://www.w3.org/ns/hydra/core#Collection",
            "supportedOperation": [
                {
                    "@id": "_:dummyclasses_retrieve",
                    "@type": "http://schema.org/FindAction",
                    "description": "Retrieves all the members of dummyclasses",
                    "expects": "null",
                    "expectsHeader": [],
                    "method": "GET",
                    "possibleStatus": [],
                    "returns": "http://hydrus.com/api/vocab?resource=dummyClass",
                    "returnsHeader": []
                },
                {
                    "@id": "_:dummyclasses_create",
                    "@type": "http://schema.org/AddAction",
                    "description": "Create new member in dummyclasses",
                    "expects": "http://hydrus.com/api/vocab?resource=dummyClass",
                    "expectsHeader": [],
                    "method": "PUT",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "A new member in dummyclasses created",
                            "statusCode": 201,
                            "title": ""
                        }
                    ],
                    "returns": "http://hydrus.com/api/vocab?resource=dummyClass",
                    "returnsHeader": []
                },
                {
                    "@id": "_:dummyclasses_update",
                    "@type": "http://schema.org/UpdateAction",
                    "description": "Update member of  dummyclasses ",
                    "expects": "http://hydrus.com/api/vocab?resource=dummyClass",
                    "expectsHeader": [],
                    "method": "POST",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "If the entity was updatedfrom dummyclasses.",
                            "statusCode": 200,
                            "title": ""
                        }
                    ],
                    "returns": "http://hydrus.com/api/vocab?resource=dummyClass",
                    "returnsHeader": []
                },
                {
                    "@id": "_:dummyclasses_delete",
                    "@type": "http://schema.org/DeleteAction",
                    "description": "Delete member of dummyclasses ",
                    "expects": "http://hydrus.com/api/vocab?resource=dummyClass",
                    "expectsHeader": [],
                    "method": "DELETE",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "If entity was deletedsuccessfully from dummyclasses.",
                            "statusCode": 200,
                            "title": ""
                        }
                    ],
                    "returns": "http://hydrus.com/api/vocab?resource=dummyClass",
                    "returnsHeader": []
                }
            ],
            "supportedProperty": [
                {
                    "@type": "SupportedProperty",
                    "description": "The members of dummyclasses",
                    "property": "http://www.w3.org/ns/hydra/core#member",
                    "readable": "true",
                    "required": "false",
                    "title": "members",
                    "writeable": "true"
                }
            ],
            "title": "dummyclasses"
        },
        {
            "@id": "http://hydrus.com/api#EntryPoint",
            "@type": "hydra:Class",
            "description": "The main entry point or homepage of the API.",
            "supportedOperation": [
                {
                    "@id": "_:entry_point",
                    "@type": "http://hydrus.com//api#EntryPoint",
                    "description": "The APIs main entry point.",
                    "expects": "null",
                    "expectsHeader": [],
                    "method": "GET",
                    "possibleStatus": [],
                    "returns": "null",
                    "returnsHeader": []
                }
            ],
            "supportedProperty": [
                {
                    "hydra:description": "The singleClass Class",
                    "hydra:title": "singleclass",
                    "property": {
                        "@id": "http://hydrus.com/api/vocab?resource=EntryPoint/singleClass",
                        "@type": "hydra:Link",
                        "description": "A non collection class",
                        "domain": "http://hydrus.com/api/vocab?resource=EntryPoint",
                        "label": "singleClass",
                        "range": "http://hydrus.com/api/vocab?resource=singleClass",
                        "supportedOperation": [
                            {
                                "@id": "updateclass",
                                "@type": "http://schema.org/UpdateAction",
                                "description": "null",
                                "expects": "http://hydrus.com/api/vocab?resource=singleClass",
                                "expectsHeader": [],
                                "label": "UpdateClass",
                                "method": "POST",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "singleClass changed.",
                                        "statusCode": 200,
                                        "title": ""
                                    }
                                ],
                                "returns": "null",
                                "returnsHeader": []
                            },
                            {
                                "@id": "deleteclass",
                                "@type": "http://schema.org/DeleteAction",
                                "description": "null",
                                "expects": "null",
                                "expectsHeader": [],
                                "label": "DeleteClass",
                                "method": "DELETE",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "singleClass deleted.",
                                        "statusCode": 200,
                                        "title": ""
                                    }
                                ],
                                "returns": "null",
                                "returnsHeader": []
                            },
                            {
                                "@id": "addclass",
                                "@type": "http://schema.org/AddAction",
                                "description": "null",
                                "expects": "http://hydrus.com/api/vocab?resource=singleClass",
                                "expectsHeader": [],
                                "label": "AddClass",
                                "method": "PUT",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "singleClass successfully added.",
                                        "statusCode": 201,
                                        "title": ""
                                    }
                                ],
                                "returns": "null",
                                "returnsHeader": []
                            },
                            {
                                "@id": "getclass",
                                "@type": "http://schema.org/FindAction",
                                "description": "null",
                                "expects": "null",
                                "expectsHeader": [],
                                "label": "GetClass",
                                "method": "GET",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "singleClass returned.",
                                        "statusCode": 200,
                                        "title": ""
                                    }
                                ],
                                "returns": "http://hydrus.com/api/vocab?resource=singleClass",
                                "returnsHeader": []
                            }
                        ]
                    },
                    "readable": "true",
                    "required": "null",
                    "writeable": "false"
                },
                {
                    "hydra:description": "The anotherSingleClass Class",
                    "hydra:title": "anothersingleclass",
                    "property": {
                        "@id": "http://hydrus.com/api/vocab?resource=EntryPoint/anotherSingleClass",
                        "@type": "hydra:Link",
                        "description": "An another non collection class",
                        "domain": "http://hydrus.com/api/vocab?resource=EntryPoint",
                        "label": "anotherSingleClass",
                        "range": "http://hydrus.com/api/vocab?resource=anotherSingleClass",
                        "supportedOperation": [
                            {
                                "@id": "getclass",
                                "@type": "http://schema.org/FindAction",
                                "description": "null",
                                "expects": "null",
                                "expectsHeader": [],
                                "label": "GetClass",
                                "method": "GET",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "anotherSingleClass returned.",
                                        "statusCode": 200,
                                        "title": ""
                                    }
                                ],
                                "returns": "http://hydrus.com/api/vocab?resource=anotherSingleClass",
                                "returnsHeader": []
                            }
                        ]
                    },
                    "readable": "true",
                    "required": "null",
                    "writeable": "false"
                },
                {
                    "hydra:description": "The Extraclasses collection",
                    "hydra:title": "extraclasses",
                    "property": {
                        "@id": "http://hydrus.com/api/vocab?resource=EntryPoint/EcTest",
                        "@type": "hydra:Link",
                        "description": "The Extraclasses collection",
                        "domain": "http://hydrus.com/api/vocab?resource=EntryPoint",
                        "label": "Extraclasses",
                        "manages": {
                            "object": "http://hydrus.com/api/vocab?resource=extraClass",
                            "property": "rdf:type"
                        },
                        "range": "http://hydrus.com/api/vocab?resource=Extraclasses",
                        "supportedOperation": [
                            {
                                "@id": "_:extraclasses_retrieve",
                                "@type": "http://schema.org/FindAction",
                                "description": "Retrieves all the members of Extraclasses",
                                "expects": "null",
                                "expectsHeader": [],
                                "method": "GET",
                                "possibleStatus": [],
                                "returns": "http://hydrus.com/api/vocab?resource=extraClass",
                                "returnsHeader": []
                            },
                            {
                                "@id": "_:extraclasses_create",
                                "@type": "http://schema.org/AddAction",
                                "description": "Create new member in Extraclasses",
                                "expects": "http://hydrus.com/api/vocab?resource=extraClass",
                                "expectsHeader": [],
                                "method": "PUT",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "A new member in Extraclasses created",
                                        "statusCode": 201,
                                        "title": ""
                                    }
                                ],
                                "returns": "http://hydrus.com/api/vocab?resource=extraClass",
                                "returnsHeader": []
                            },
                            {
                                "@id": "_:extraclasses_update",
                                "@type": "http://schema.org/UpdateAction",
                                "description": "Update member of  Extraclasses ",
                                "expects": "http://hydrus.com/api/vocab?resource=extraClass",
                                "expectsHeader": [],
                                "method": "POST",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "If the entity was updatedfrom Extraclasses.",
                                        "statusCode": 200,
                                        "title": ""
                                    }
                                ],
                                "returns": "http://hydrus.com/api/vocab?resource=extraClass",
                                "returnsHeader": []
                            },
                            {
                                "@id": "_:extraclasses_delete",
                                "@type": "http://schema.org/DeleteAction",
                                "description": "Delete member of Extraclasses ",
                                "expects": "http://hydrus.com/api/vocab?resource=extraClass",
                                "expectsHeader": [],
                                "method": "DELETE",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "If entity was deletedsuccessfully from Extraclasses.",
                                        "statusCode": 200,
                                        "title": ""
                                    }
                                ],
                                "returns": "http://hydrus.com/api/vocab?resource=extraClass",
                                "returnsHeader": []
                            }
                        ]
                    },
                    "readable": "true",
                    "required": "null",
                    "writeable": "false"
                },
                {
                    "hydra:description": "The dummyclasses collection",
                    "hydra:title": "dummyclasses",
                    "property": {
                        "@id": "http://hydrus.com/api/vocab?resource=EntryPoint/DcTest",
                        "@type": "hydra:Link",
                        "description": "The dummyclasses collection",
                        "domain": "http://hydrus.com/api/vocab?resource=EntryPoint",
                        "label": "dummyclasses",
                        "manages": {
                            "object": "http://hydrus.com/api/vocab?resource=dummyClass",
                            "property": "rdf:type"
                        },
                        "range": "http://hydrus.com/api/vocab?resource=dummyclasses",
                        "supportedOperation": [
                            {
                                "@id": "_:dummyclasses_retrieve",
                                "@type": "http://schema.org/FindAction",
                                "description": "Retrieves all the members of dummyclasses",
                                "expects": "null",
                                "expectsHeader": [],
                                "method": "GET",
                                "possibleStatus": [],
                                "returns": "http://hydrus.com/api/vocab?resource=dummyClass",
                                "returnsHeader": []
                            },
                            {
                                "@id": "_:dummyclasses_create",
                                "@type": "http://schema.org/AddAction",
                                "description": "Create new member in dummyclasses",
                                "expects": "http://hydrus.com/api/vocab?resource=dummyClass",
                                "expectsHeader": [],
                                "method": "PUT",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "A new member in dummyclasses created",
                                        "statusCode": 201,
                                        "title": ""
                                    }
                                ],
                                "returns": "http://hydrus.com/api/vocab?resource=dummyClass",
                                "returnsHeader": []
                            },
                            {
                                "@id": "_:dummyclasses_update",
                                "@type": "http://schema.org/UpdateAction",
                                "description": "Update member of  dummyclasses ",
                                "expects": "http://hydrus.com/api/vocab?resource=dummyClass",
                                "expectsHeader": [],
                                "method": "POST",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "If the entity was updatedfrom dummyclasses.",
                                        "statusCode": 200,
                                        "title": ""
                                    }
                                ],
                                "returns": "http://hydrus.com/api/vocab?resource=dummyClass",
                                "returnsHeader": []
                            },
                            {
                                "@id": "_:dummyclasses_delete",
                                "@type": "http://schema.org/DeleteAction",
                                "description": "Delete member of dummyclasses ",
                                "expects": "http://hydrus.com/api/vocab?resource=dummyClass",
                                "expectsHeader": [],
                                "method": "DELETE",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "If entity was deletedsuccessfully from dummyclasses.",
                                        "statusCode": 200,
                                        "title": ""
                                    }
                                ],
                                "returns": "http://hydrus.com/api/vocab?resource=dummyClass",
                                "returnsHeader": []
                            }
                        ]
                    },
                    "readable": "true",
                    "required": "null",
                    "writeable": "false"
                }
            ],
            "title": "EntryPoint"
        }
    ],
    "title": "Title for the API Documentation"
}



"""
Generated API Documentation for Server API using server_doc_gen.py."""

doc = {
    "@context": {
        "ApiDocumentation": "hydra:ApiDocumentation",
        "description": "hydra:description",
        "domain": {
            "@id": "rdfs:domain",
            "@type": "@id"
        },
        "entrypoint": {
            "@id": "hydra:entrypoint",
            "@type": "@id"
        },
        "expects": {
            "@id": "hydra:expects",
            "@type": "@id"
        },
        "expectsHeader": "hydra:expectsHeader",
        "hydra": "http://www.w3.org/ns/hydra/core#",
        "label": "rdfs:label",
        "manages": "hydra:manages",
        "method": "hydra:method",
        "object": {
            "@id": "hydra:object",
            "@type": "@id"
        },
        "possibleStatus": "hydra:possibleStatus",
        "property": {
            "@id": "hydra:property",
            "@type": "@id"
        },
        "range": {
            "@id": "rdfs:range",
            "@type": "@id"
        },
        "rdf": "http://www.w3.org/1999/02/22-rdf-syntax-ns#",
        "rdfs": "http://www.w3.org/2000/01/rdf-schema#",
        "readable": "hydra:readable",
        "required": "hydra:required",
        "returns": {
            "@id": "hydra:returns",
            "@type": "@id"
        },
        "returnsHeader": "hydra:returnsHeader",
        "search": "hydra:search",
        "statusCode": "hydra:statusCode",
        "subClassOf": {
            "@id": "rdfs:subClassOf",
            "@type": "@id"
        },
        "subject": {
            "@id": "hydra:subject",
            "@type": "@id"
        },
        "supportedClass": "hydra:supportedClass",
        "supportedOperation": "hydra:supportedOperation",
        "supportedProperty": "hydra:supportedProperty",
        "title": "hydra:title",
        "writeable": "hydra:writeable"
    },
    "@id": "http://localhost:8080/api/vocab",
    "@type": "ApiDocumentation",
    "description": "API Documentation for the server side system",
    "entrypoint": "http://localhost:8080/api",
    "possibleStatus": [],
    "supportedClass": [
        {
            "@id": "http://localhost:8080/api/vocab?resource=Drone",
            "@type": "hydra:Class",
            "description": "Class for a drone",
            "supportedOperation": [
                {
                    "@type": "http://schema.org/UpdateAction",
                    "expects": "http://localhost:8080/api/vocab?resource=Drone",
                    "expectsHeader": [],
                    "method": "POST",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "Drone updated",
                            "statusCode": 200,
                            "title": ""
                        }
                    ],
                    "returns": "null",
                    "returnsHeader": [],
                    "title": "SubmitDrone"
                },
                {
                    "@type": "http://schema.org/AddAction",
                    "expects": "http://localhost:8080/api/vocab?resource=Drone",
                    "expectsHeader": [],
                    "method": "PUT",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "Drone added",
                            "statusCode": 200,
                            "title": ""
                        }
                    ],
                    "returns": "null",
                    "returnsHeader": [],
                    "title": "CreateDrone"
                },
                {
                    "@type": "http://schema.org/FindAction",
                    "expects": "null",
                    "expectsHeader": [],
                    "method": "GET",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "Drone not found",
                            "statusCode": 404,
                            "title": ""
                        },
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "Drone Returned",
                            "statusCode": 200,
                            "title": ""
                        }
                    ],
                    "returns": "http://localhost:8080/api/vocab?resource=Drone",
                    "returnsHeader": [],
                    "title": "GetDrone"
                }
            ],
            "supportedProperty": [
                {
                    "@type": "SupportedProperty",
                    "property": "http://localhost:8080/api/vocab?resource=State",
                    "readable": "false",
                    "required": "true",
                    "title": "DroneState",
                    "writeable": "false"
                },
                {
                    "@type": "SupportedProperty",
                    "property": "http://schema.org/name",
                    "readable": "false",
                    "required": "true",
                    "title": "name",
                    "writeable": "false"
                },
                {
                    "@type": "SupportedProperty",
                    "property": "http://schema.org/model",
                    "readable": "false",
                    "required": "true",
                    "title": "model",
                    "writeable": "false"
                },
                {
                    "@type": "SupportedProperty",
                    "property": "http://auto.schema.org/speed",
                    "readable": "false",
                    "required": "true",
                    "title": "MaxSpeed",
                    "writeable": "false"
                },
                {
                    "@type": "SupportedProperty",
                    "property": "http://schema.org/device",
                    "readable": "false",
                    "required": "true",
                    "title": "Sensor",
                    "writeable": "false"
                }
            ],
            "title": "Drone"
        },
        {
            "@id": "http://localhost:8080/api/vocab?resource=State",
            "@type": "hydra:Class",
            "description": "Class for drone state objects",
            "supportedOperation": [
                {
                    "@type": "http://schema.org/FindAction",
                    "expects": "null",
                    "expectsHeader": [],
                    "method": "GET",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "State not found",
                            "statusCode": 404,
                            "title": ""
                        },
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "State Returned",
                            "statusCode": 200,
                            "title": ""
                        }
                    ],
                    "returns": "http://localhost:8080/api/vocab?resource=State",
                    "returnsHeader": [],
                    "title": "GetState"
                }
            ],
            "supportedProperty": [
                {
                    "@type": "SupportedProperty",
                    "property": "http://auto.schema.org/speed",
                    "readable": "false",
                    "required": "true",
                    "title": "Speed",
                    "writeable": "false"
                },
                {
                    "@type": "SupportedProperty",
                    "property": "http://schema.org/geo",
                    "readable": "false",
                    "required": "true",
                    "title": "Position",
                    "writeable": "false"
                },
                {
                    "@type": "SupportedProperty",
                    "property": "http://schema.org/Property",
                    "readable": "false",
                    "required": "true",
                    "title": "Direction",
                    "writeable": "false"
                },
                {
                    "@type": "SupportedProperty",
                    "property": "http://schema.org/fuelCapacity",
                    "readable": "false",
                    "required": "true",
                    "title": "Battery",
                    "writeable": "false"
                },
                {
                    "@type": "SupportedProperty",
                    "property": "https://schema.org/status",
                    "readable": "false",
                    "required": "true",
                    "title": "SensorStatus",
                    "writeable": "false"
                },
                {
                    "@type": "SupportedProperty",
                    "property": "http://schema.org/identifier",
                    "readable": "false",
                    "required": "true",
                    "title": "DroneID",
                    "writeable": "false"
                }
            ],
            "title": "State"
        },
        {
            "@id": "http://localhost:8080/api/vocab?resource=Datastream",
            "@type": "hydra:Class",
            "description": "Class for a datastream entry",
            "supportedOperation": [
                {
                    "@type": "http://schema.org/FindAction",
                    "expects": "null",
                    "expectsHeader": [],
                    "method": "GET",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "Data not found",
                            "statusCode": 404,
                            "title": ""
                        },
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "Data returned",
                            "statusCode": 200,
                            "title": ""
                        }
                    ],
                    "returns": "http://localhost:8080/api/vocab?resource=Datastream",
                    "returnsHeader": [],
                    "title": "ReadDatastream"
                },
                {
                    "@type": "http://schema.org/UpdateAction",
                    "expects": "http://localhost:8080/api/vocab?resource=Datastream",
                    "expectsHeader": [],
                    "method": "POST",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "Data updated",
                            "statusCode": 200,
                            "title": ""
                        }
                    ],
                    "returns": "null",
                    "returnsHeader": [],
                    "title": "UpdateDatastream"
                },
                {
                    "@type": "http://schema.org/DeleteAction",
                    "expects": "null",
                    "expectsHeader": [],
                    "method": "DELETE",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "Data deleted",
                            "statusCode": 200,
                            "title": ""
                        }
                    ],
                    "returns": "null",
                    "returnsHeader": [],
                    "title": "DeleteDatastream"
                }
            ],
            "supportedProperty": [
                {
                    "@type": "SupportedProperty",
                    "property": "http://schema.org/QuantitativeValue",
                    "readable": "false",
                    "required": "true",
                    "title": "Temperature",
                    "writeable": "false"
                },
                {
                    "@type": "SupportedProperty",
                    "property": "http://schema.org/identifier",
                    "readable": "false",
                    "required": "true",
                    "title": "DroneID",
                    "writeable": "false"
                },
                {
                    "@type": "SupportedProperty",
                    "property": "http://schema.org/geo",
                    "readable": "false",
                    "required": "true",
                    "title": "Position",
                    "writeable": "false"
                }
            ],
            "title": "Datastream"
        },
        {
            "@id": "http://localhost:8080/api/vocab?resource=LogEntry",
            "@type": "hydra:Class",
            "description": "Class for a log entry",
            "supportedOperation": [
                {
                    "@type": "http://schema.org/FindAction",
                    "expects": "null",
                    "expectsHeader": [],
                    "method": "GET",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "Log entry not found",
                            "statusCode": 404,
                            "title": ""
                        },
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "Log entry returned",
                            "statusCode": 200,
                            "title": ""
                        }
                    ],
                    "returns": "http://localhost:8080/api/vocab?resource=LogEntry",
                    "returnsHeader": [],
                    "title": "GetLog"
                },
                {
                    "@type": "http://schema.org/AddAction",
                    "expects": "http://localhost:8080/api/vocab?resource=LogEntry",
                    "expectsHeader": [],
                    "method": "PUT",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "Log entry created",
                            "statusCode": 201,
                            "title": ""
                        }
                    ],
                    "returns": "null",
                    "returnsHeader": [],
                    "title": "AddLog"
                }
            ],
            "supportedProperty": [
                {
                    "@type": "SupportedProperty",
                    "property": "http://schema.org/identifier",
                    "readable": "true",
                    "required": "false",
                    "title": "DroneID",
                    "writeable": "true"
                },
                {
                    "@type": "SupportedProperty",
                    "property": "http://schema.org/UpdateAction",
                    "readable": "false",
                    "required": "false",
                    "title": "Update",
                    "writeable": "true"
                },
                {
                    "@type": "SupportedProperty",
                    "property": "http://schema.org/ReplyAction",
                    "readable": "false",
                    "required": "false",
                    "title": "Get",
                    "writeable": "true"
                },
                {
                    "@type": "SupportedProperty",
                    "property": "http://schema.org/SendAction",
                    "readable": "false",
                    "required": "false",
                    "title": "Send",
                    "writeable": "true"
                },
                {
                    "@type": "SupportedProperty",
                    "property": "http://localhost:8080/api/vocab?resource=State",
                    "readable": "false",
                    "required": "false",
                    "title": "State",
                    "writeable": "true"
                },
                {
                    "@type": "SupportedProperty",
                    "property": "http://localhost:8080/api/vocab?resource=Datastream",
                    "readable": "false",
                    "required": "false",
                    "title": "Data",
                    "writeable": "true"
                },
                {
                    "@type": "SupportedProperty",
                    "property": "http://localhost:8080/api/vocab?resource=Command",
                    "readable": "false",
                    "required": "false",
                    "title": "Command",
                    "writeable": "true"
                }
            ],
            "title": "LogEntry"
        },
        {
            "@id": "http://localhost:8080/api/vocab?resource=Area",
            "@type": "hydra:Class",
            "description": "Class for Area of Interest of the server",
            "supportedOperation": [
                {
                    "@type": "http://schema.org/UpdateAction",
                    "expects": "http://localhost:8080/api/vocab?resource=Area",
                    "expectsHeader": [],
                    "method": "POST",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "Area of interest changed",
                            "statusCode": 200,
                            "title": ""
                        }
                    ],
                    "returns": "null",
                    "returnsHeader": [],
                    "title": "UpdateArea"
                },
                {
                    "@type": "http://schema.org/FindAction",
                    "expects": "null",
                    "expectsHeader": [],
                    "method": "GET",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "Area of interest not found",
                            "statusCode": 200,
                            "title": ""
                        },
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "Area of interest returned",
                            "statusCode": 200,
                            "title": ""
                        }
                    ],
                    "returns": "http://localhost:8080/api/vocab?resource=Area",
                    "returnsHeader": [],
                    "title": "GetArea"
                }
            ],
            "supportedProperty": [
                {
                    "@type": "SupportedProperty",
                    "property": "http://schema.org/geo",
                    "readable": "false",
                    "required": "true",
                    "title": "TopLeft",
                    "writeable": "false"
                },
                {
                    "@type": "SupportedProperty",
                    "property": "http://schema.org/geo",
                    "readable": "false",
                    "required": "true",
                    "title": "BottomRight",
                    "writeable": "false"
                }
            ],
            "title": "Area"
        },
        {
            "@id": "http://localhost:8080/api/vocab?resource=Command",
            "@type": "hydra:Class",
            "description": "Class for drone commands",
            "supportedOperation": [
                {
                    "@type": "http://schema.org/FindAction",
                    "expects": "null",
                    "expectsHeader": [],
                    "method": "GET",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "Command not found",
                            "statusCode": 404,
                            "title": ""
                        },
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "Command Returned",
                            "statusCode": 200,
                            "title": ""
                        }
                    ],
                    "returns": "http://localhost:8080/api/vocab?resource=Command",
                    "returnsHeader": [],
                    "title": "GetCommand"
                },
                {
                    "@type": "http://schema.org/AddAction",
                    "expects": "http://localhost:8080/api/vocab?resource=Command",
                    "expectsHeader": [],
                    "method": "PUT",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "Command added",
                            "statusCode": 201,
                            "title": ""
                        }
                    ],
                    "returns": "null",
                    "returnsHeader": [],
                    "title": "AddCommand"
                },
                {
                    "@type": "http://schema.org/DeleteAction",
                    "expects": "null",
                    "expectsHeader": [],
                    "method": "DELETE",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "Command deleted",
                            "statusCode": 201,
                            "title": ""
                        }
                    ],
                    "returns": "null",
                    "returnsHeader": [],
                    "title": "DeleteCommand"
                }
            ],
            "supportedProperty": [
                {
                    "@type": "SupportedProperty",
                    "property": "http://schema.org/identifier",
                    "readable": "false",
                    "required": "true",
                    "title": "DroneID",
                    "writeable": "false"
                },
                {
                    "@type": "SupportedProperty",
                    "property": "http://localhost:8080/api/vocab?resource=State",
                    "readable": "false",
                    "required": "true",
                    "title": "State",
                    "writeable": "false"
                }
            ],
            "title": "Command"
        },
        {
            "@id": "http://localhost:8080/api/vocab?resource=Message",
            "@type": "hydra:Class",
            "description": "Class for messages received by the GUI interface",
            "supportedOperation": [
                {
                    "@type": "http://schema.org/FindAction",
                    "expects": "null",
                    "expectsHeader": [],
                    "method": "GET",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "Message not found",
                            "statusCode": 200,
                            "title": ""
                        },
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "Message returned",
                            "statusCode": 200,
                            "title": ""
                        }
                    ],
                    "returns": "http://localhost:8080/api/vocab?resource=Message",
                    "returnsHeader": [],
                    "title": "GetMessage"
                },
                {
                    "@type": "http://schema.org/DeleteAction",
                    "expects": "null",
                    "expectsHeader": [],
                    "method": "DELETE",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "Message deleted",
                            "statusCode": 200,
                            "title": ""
                        }
                    ],
                    "returns": "null",
                    "returnsHeader": [],
                    "title": "DeleteMessage"
                }
            ],
            "supportedProperty": [
                {
                    "@type": "SupportedProperty",
                    "property": "http://schema.org/Text",
                    "readable": "false",
                    "required": "true",
                    "title": "MessageString",
                    "writeable": "false"
                }
            ],
            "title": "Message"
        },
        {
            "@id": "http://www.w3.org/ns/hydra/core#Resource",
            "@type": "hydra:Class",
            "description": "null",
            "supportedOperation": [],
            "supportedProperty": [],
            "title": "Resource"
        },
        {
            "@id": "http://www.w3.org/ns/hydra/core#Collection",
            "@type": "hydra:Class",
            "description": "null",
            "supportedOperation": [],
            "supportedProperty": [
                {
                    "@type": "SupportedProperty",
                    "property": "http://www.w3.org/ns/hydra/core#member",
                    "readable": "false",
                    "required": "null",
                    "title": "members",
                    "writeable": "false"
                }
            ],
            "title": "Collection"
        },
        {
            "@id": "http://localhost:8080/api/vocab?resource=DroneCollection",
            "@type": "Collection",
            "description": "A collection of drones",
            "manages": {
                "object": "http://localhost:8080/api/vocab?resource=Drone",
                "property": "rdfs:type"
            },
            "subClassOf": "http://www.w3.org/ns/hydra/core#Collection",
            "supportedOperation": [
                {
                    "@id": "_:DroneCollection_retrieve",
                    "@type": "http://schema.org/FindAction",
                    "description": "Retrieves all the members of DroneCollection",
                    "expects": "null",
                    "expectsHeader": [],
                    "method": "GET",
                    "possibleStatus": [],
                    "returns": "http://localhost:8080/api/vocab?resource=Drone",
                    "returnsHeader": []
                },
                {
                    "@id": "_:DroneCollection_create",
                    "@type": "http://schema.org/AddAction",
                    "description": "Create new member in DroneCollection",
                    "expects": "http://localhost:8080/api/vocab?resource=Drone",
                    "expectsHeader": [],
                    "method": "PUT",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "A new member in DroneCollection created",
                            "statusCode": 201,
                            "title": ""
                        }
                    ],
                    "returns": "http://localhost:8080/api/vocab?resource=Drone",
                    "returnsHeader": []
                },
                {
                    "@id": "_:DroneCollection_update",
                    "@type": "http://schema.org/UpdateAction",
                    "description": "Update member of  DroneCollection ",
                    "expects": "http://localhost:8080/api/vocab?resource=Drone",
                    "expectsHeader": [],
                    "method": "POST",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "If the entity was updatedfrom DroneCollection.",
                            "statusCode": 200,
                            "title": ""
                        }
                    ],
                    "returns": "http://localhost:8080/api/vocab?resource=Drone",
                    "returnsHeader": []
                },
                {
                    "@id": "_:DroneCollection_delete",
                    "@type": "http://schema.org/DeleteAction",
                    "description": "Delete member of DroneCollection ",
                    "expects": "http://localhost:8080/api/vocab?resource=Drone",
                    "expectsHeader": [],
                    "method": "DELETE",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "If entity was deletedsuccessfully from DroneCollection.",
                            "statusCode": 200,
                            "title": ""
                        }
                    ],
                    "returns": "http://localhost:8080/api/vocab?resource=Drone",
                    "returnsHeader": []
                }
            ],
            "supportedProperty": [
                {
                    "@type": "SupportedProperty",
                    "description": "The members of DroneCollection",
                    "property": "http://www.w3.org/ns/hydra/core#member",
                    "readable": "true",
                    "required": "false",
                    "title": "members",
                    "writeable": "true"
                }
            ],
            "title": "DroneCollection"
        },
        {
            "@id": "http://localhost:8080/api/vocab?resource=StateCollection",
            "@type": "Collection",
            "description": "A collection of states",
            "manages": {
                "object": "http://localhost:8080/api/vocab?resource=State",
                "property": "rdfs:type"
            },
            "subClassOf": "http://www.w3.org/ns/hydra/core#Collection",
            "supportedOperation": [
                {
                    "@id": "_:StateCollection_retrieve",
                    "@type": "http://schema.org/FindAction",
                    "description": "Retrieves all the members of StateCollection",
                    "expects": "null",
                    "expectsHeader": [],
                    "method": "GET",
                    "possibleStatus": [],
                    "returns": "http://localhost:8080/api/vocab?resource=State",
                    "returnsHeader": []
                },
                {
                    "@id": "_:StateCollection_create",
                    "@type": "http://schema.org/AddAction",
                    "description": "Create new member in StateCollection",
                    "expects": "http://localhost:8080/api/vocab?resource=State",
                    "expectsHeader": [],
                    "method": "PUT",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "A new member in StateCollection created",
                            "statusCode": 201,
                            "title": ""
                        }
                    ],
                    "returns": "http://localhost:8080/api/vocab?resource=State",
                    "returnsHeader": []
                },
                {
                    "@id": "_:StateCollection_update",
                    "@type": "http://schema.org/UpdateAction",
                    "description": "Update member of  StateCollection ",
                    "expects": "http://localhost:8080/api/vocab?resource=State",
                    "expectsHeader": [],
                    "method": "POST",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "If the entity was updatedfrom StateCollection.",
                            "statusCode": 200,
                            "title": ""
                        }
                    ],
                    "returns": "http://localhost:8080/api/vocab?resource=State",
                    "returnsHeader": []
                },
                {
                    "@id": "_:StateCollection_delete",
                    "@type": "http://schema.org/DeleteAction",
                    "description": "Delete member of StateCollection ",
                    "expects": "http://localhost:8080/api/vocab?resource=State",
                    "expectsHeader": [],
                    "method": "DELETE",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "If entity was deletedsuccessfully from StateCollection.",
                            "statusCode": 200,
                            "title": ""
                        }
                    ],
                    "returns": "http://localhost:8080/api/vocab?resource=State",
                    "returnsHeader": []
                }
            ],
            "supportedProperty": [
                {
                    "@type": "SupportedProperty",
                    "description": "The members of StateCollection",
                    "property": "http://www.w3.org/ns/hydra/core#member",
                    "readable": "true",
                    "required": "false",
                    "title": "members",
                    "writeable": "true"
                }
            ],
            "title": "StateCollection"
        },
        {
            "@id": "http://localhost:8080/api/vocab?resource=DatastreamCollection",
            "@type": "Collection",
            "description": "A collection of datastream",
            "manages": {
                "object": "http://localhost:8080/api/vocab?resource=Datastream",
                "property": "rdfs:type"
            },
            "subClassOf": "http://www.w3.org/ns/hydra/core#Collection",
            "supportedOperation": [
                {
                    "@id": "_:DatastreamCollection_retrieve",
                    "@type": "http://schema.org/FindAction",
                    "description": "Retrieves all the members of DatastreamCollection",
                    "expects": "null",
                    "expectsHeader": [],
                    "method": "GET",
                    "possibleStatus": [],
                    "returns": "http://localhost:8080/api/vocab?resource=Datastream",
                    "returnsHeader": []
                },
                {
                    "@id": "_:DatastreamCollection_create",
                    "@type": "http://schema.org/AddAction",
                    "description": "Create new member in DatastreamCollection",
                    "expects": "http://localhost:8080/api/vocab?resource=Datastream",
                    "expectsHeader": [],
                    "method": "PUT",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "A new member in DatastreamCollection created",
                            "statusCode": 201,
                            "title": ""
                        }
                    ],
                    "returns": "http://localhost:8080/api/vocab?resource=Datastream",
                    "returnsHeader": []
                },
                {
                    "@id": "_:DatastreamCollection_update",
                    "@type": "http://schema.org/UpdateAction",
                    "description": "Update member of  DatastreamCollection ",
                    "expects": "http://localhost:8080/api/vocab?resource=Datastream",
                    "expectsHeader": [],
                    "method": "POST",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "If the entity was updatedfrom DatastreamCollection.",
                            "statusCode": 200,
                            "title": ""
                        }
                    ],
                    "returns": "http://localhost:8080/api/vocab?resource=Datastream",
                    "returnsHeader": []
                },
                {
                    "@id": "_:DatastreamCollection_delete",
                    "@type": "http://schema.org/DeleteAction",
                    "description": "Delete member of DatastreamCollection ",
                    "expects": "http://localhost:8080/api/vocab?resource=Datastream",
                    "expectsHeader": [],
                    "method": "DELETE",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "If entity was deletedsuccessfully from DatastreamCollection.",
                            "statusCode": 200,
                            "title": ""
                        }
                    ],
                    "returns": "http://localhost:8080/api/vocab?resource=Datastream",
                    "returnsHeader": []
                }
            ],
            "supportedProperty": [
                {
                    "@type": "SupportedProperty",
                    "description": "The members of DatastreamCollection",
                    "property": "http://www.w3.org/ns/hydra/core#member",
                    "readable": "true",
                    "required": "false",
                    "title": "members",
                    "writeable": "true"
                }
            ],
            "title": "DatastreamCollection"
        },
        {
            "@id": "http://localhost:8080/api/vocab?resource=LogEntryCollection",
            "@type": "Collection",
            "description": "A collection of logs",
            "manages": {
                "object": "http://localhost:8080/api/vocab?resource=LogEntry",
                "property": "rdfs:type"
            },
            "subClassOf": "http://www.w3.org/ns/hydra/core#Collection",
            "supportedOperation": [
                {
                    "@id": "_:LogEntryCollection_retrieve",
                    "@type": "http://schema.org/FindAction",
                    "description": "Retrieves all the members of LogEntryCollection",
                    "expects": "null",
                    "expectsHeader": [],
                    "method": "GET",
                    "possibleStatus": [],
                    "returns": "http://localhost:8080/api/vocab?resource=LogEntry",
                    "returnsHeader": []
                },
                {
                    "@id": "_:LogEntryCollection_create",
                    "@type": "http://schema.org/AddAction",
                    "description": "Create new member in LogEntryCollection",
                    "expects": "http://localhost:8080/api/vocab?resource=LogEntry",
                    "expectsHeader": [],
                    "method": "PUT",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "A new member in LogEntryCollection created",
                            "statusCode": 201,
                            "title": ""
                        }
                    ],
                    "returns": "http://localhost:8080/api/vocab?resource=LogEntry",
                    "returnsHeader": []
                },
                {
                    "@id": "_:LogEntryCollection_update",
                    "@type": "http://schema.org/UpdateAction",
                    "description": "Update member of  LogEntryCollection ",
                    "expects": "http://localhost:8080/api/vocab?resource=LogEntry",
                    "expectsHeader": [],
                    "method": "POST",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "If the entity was updatedfrom LogEntryCollection.",
                            "statusCode": 200,
                            "title": ""
                        }
                    ],
                    "returns": "http://localhost:8080/api/vocab?resource=LogEntry",
                    "returnsHeader": []
                },
                {
                    "@id": "_:LogEntryCollection_delete",
                    "@type": "http://schema.org/DeleteAction",
                    "description": "Delete member of LogEntryCollection ",
                    "expects": "http://localhost:8080/api/vocab?resource=LogEntry",
                    "expectsHeader": [],
                    "method": "DELETE",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "If entity was deletedsuccessfully from LogEntryCollection.",
                            "statusCode": 200,
                            "title": ""
                        }
                    ],
                    "returns": "http://localhost:8080/api/vocab?resource=LogEntry",
                    "returnsHeader": []
                }
            ],
            "supportedProperty": [
                {
                    "@type": "SupportedProperty",
                    "description": "The members of LogEntryCollection",
                    "property": "http://www.w3.org/ns/hydra/core#member",
                    "readable": "true",
                    "required": "false",
                    "title": "members",
                    "writeable": "true"
                }
            ],
            "title": "LogEntryCollection"
        },
        {
            "@id": "http://localhost:8080/api/vocab?resource=CommandCollection",
            "@type": "Collection",
            "description": "A collection of commands",
            "manages": {
                "object": "http://localhost:8080/api/vocab?resource=Command",
                "property": "rdfs:type"
            },
            "subClassOf": "http://www.w3.org/ns/hydra/core#Collection",
            "supportedOperation": [
                {
                    "@id": "_:CommandCollection_retrieve",
                    "@type": "http://schema.org/FindAction",
                    "description": "Retrieves all the members of CommandCollection",
                    "expects": "null",
                    "expectsHeader": [],
                    "method": "GET",
                    "possibleStatus": [],
                    "returns": "http://localhost:8080/api/vocab?resource=Command",
                    "returnsHeader": []
                },
                {
                    "@id": "_:CommandCollection_create",
                    "@type": "http://schema.org/AddAction",
                    "description": "Create new member in CommandCollection",
                    "expects": "http://localhost:8080/api/vocab?resource=Command",
                    "expectsHeader": [],
                    "method": "PUT",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "A new member in CommandCollection created",
                            "statusCode": 201,
                            "title": ""
                        }
                    ],
                    "returns": "http://localhost:8080/api/vocab?resource=Command",
                    "returnsHeader": []
                },
                {
                    "@id": "_:CommandCollection_update",
                    "@type": "http://schema.org/UpdateAction",
                    "description": "Update member of  CommandCollection ",
                    "expects": "http://localhost:8080/api/vocab?resource=Command",
                    "expectsHeader": [],
                    "method": "POST",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "If the entity was updatedfrom CommandCollection.",
                            "statusCode": 200,
                            "title": ""
                        }
                    ],
                    "returns": "http://localhost:8080/api/vocab?resource=Command",
                    "returnsHeader": []
                },
                {
                    "@id": "_:CommandCollection_delete",
                    "@type": "http://schema.org/DeleteAction",
                    "description": "Delete member of CommandCollection ",
                    "expects": "http://localhost:8080/api/vocab?resource=Command",
                    "expectsHeader": [],
                    "method": "DELETE",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "If entity was deletedsuccessfully from CommandCollection.",
                            "statusCode": 200,
                            "title": ""
                        }
                    ],
                    "returns": "http://localhost:8080/api/vocab?resource=Command",
                    "returnsHeader": []
                }
            ],
            "supportedProperty": [
                {
                    "@type": "SupportedProperty",
                    "description": "The members of CommandCollection",
                    "property": "http://www.w3.org/ns/hydra/core#member",
                    "readable": "true",
                    "required": "false",
                    "title": "members",
                    "writeable": "true"
                }
            ],
            "title": "CommandCollection"
        },
        {
            "@id": "http://localhost:8080/api/vocab?resource=MessageCollection",
            "@type": "Collection",
            "description": "A collection of messages",
            "manages": {
                "object": "http://localhost:8080/api/vocab?resource=Message",
                "property": "rdfs:type"
            },
            "subClassOf": "http://www.w3.org/ns/hydra/core#Collection",
            "supportedOperation": [
                {
                    "@id": "_:MessageCollection_retrieve",
                    "@type": "http://schema.org/FindAction",
                    "description": "Retrieves all the members of MessageCollection",
                    "expects": "null",
                    "expectsHeader": [],
                    "method": "GET",
                    "possibleStatus": [],
                    "returns": "http://localhost:8080/api/vocab?resource=Message",
                    "returnsHeader": []
                },
                {
                    "@id": "_:MessageCollection_create",
                    "@type": "http://schema.org/AddAction",
                    "description": "Create new member in MessageCollection",
                    "expects": "http://localhost:8080/api/vocab?resource=Message",
                    "expectsHeader": [],
                    "method": "PUT",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "A new member in MessageCollection created",
                            "statusCode": 201,
                            "title": ""
                        }
                    ],
                    "returns": "http://localhost:8080/api/vocab?resource=Message",
                    "returnsHeader": []
                },
                {
                    "@id": "_:MessageCollection_update",
                    "@type": "http://schema.org/UpdateAction",
                    "description": "Update member of  MessageCollection ",
                    "expects": "http://localhost:8080/api/vocab?resource=Message",
                    "expectsHeader": [],
                    "method": "POST",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "If the entity was updatedfrom MessageCollection.",
                            "statusCode": 200,
                            "title": ""
                        }
                    ],
                    "returns": "http://localhost:8080/api/vocab?resource=Message",
                    "returnsHeader": []
                },
                {
                    "@id": "_:MessageCollection_delete",
                    "@type": "http://schema.org/DeleteAction",
                    "description": "Delete member of MessageCollection ",
                    "expects": "http://localhost:8080/api/vocab?resource=Message",
                    "expectsHeader": [],
                    "method": "DELETE",
                    "possibleStatus": [
                        {
                            "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                            "@type": "Status",
                            "description": "If entity was deletedsuccessfully from MessageCollection.",
                            "statusCode": 200,
                            "title": ""
                        }
                    ],
                    "returns": "http://localhost:8080/api/vocab?resource=Message",
                    "returnsHeader": []
                }
            ],
            "supportedProperty": [
                {
                    "@type": "SupportedProperty",
                    "description": "The members of MessageCollection",
                    "property": "http://www.w3.org/ns/hydra/core#member",
                    "readable": "true",
                    "required": "false",
                    "title": "members",
                    "writeable": "true"
                }
            ],
            "title": "MessageCollection"
        },
        {
            "@id": "http://localhost:8080/api#EntryPoint",
            "@type": "hydra:Class",
            "description": "The main entry point or homepage of the API.",
            "supportedOperation": [
                {
                    "@id": "_:entry_point",
                    "@type": "http://localhost:8080//api#EntryPoint",
                    "description": "The APIs main entry point.",
                    "expects": "null",
                    "expectsHeader": [],
                    "method": "GET",
                    "possibleStatus": [],
                    "returns": "null",
                    "returnsHeader": []
                }
            ],
            "supportedProperty": [
                {
                    "hydra:description": "The Drone Class",
                    "hydra:title": "drone",
                    "property": {
                        "@id": "http://localhost:8080/api/vocab?resource=EntryPoint/Drone",
                        "@type": "hydra:Link",
                        "description": "Class for a drone",
                        "domain": "http://localhost:8080/api/vocab?resource=EntryPoint",
                        "label": "Drone",
                        "range": "http://localhost:8080/api/vocab?resource=Drone",
                        "supportedOperation": [
                            {
                                "@id": "submitdrone",
                                "@type": "http://schema.org/UpdateAction",
                                "description": "null",
                                "expects": "http://localhost:8080/api/vocab?resource=Drone",
                                "expectsHeader": [],
                                "label": "SubmitDrone",
                                "method": "POST",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "Drone updated",
                                        "statusCode": 200,
                                        "title": ""
                                    }
                                ],
                                "returns": "null",
                                "returnsHeader": []
                            },
                            {
                                "@id": "createdrone",
                                "@type": "http://schema.org/AddAction",
                                "description": "null",
                                "expects": "http://localhost:8080/api/vocab?resource=Drone",
                                "expectsHeader": [],
                                "label": "CreateDrone",
                                "method": "PUT",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "Drone added",
                                        "statusCode": 200,
                                        "title": ""
                                    }
                                ],
                                "returns": "null",
                                "returnsHeader": []
                            },
                            {
                                "@id": "getdrone",
                                "@type": "http://schema.org/FindAction",
                                "description": "null",
                                "expects": "null",
                                "expectsHeader": [],
                                "label": "GetDrone",
                                "method": "GET",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "Drone not found",
                                        "statusCode": 404,
                                        "title": ""
                                    },
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "Drone Returned",
                                        "statusCode": 200,
                                        "title": ""
                                    }
                                ],
                                "returns": "http://localhost:8080/api/vocab?resource=Drone",
                                "returnsHeader": []
                            }
                        ]
                    },
                    "readable": "true",
                    "required": "null",
                    "writeable": "false"
                },
                {
                    "hydra:description": "The State Class",
                    "hydra:title": "state",
                    "property": {
                        "@id": "http://localhost:8080/api/vocab?resource=EntryPoint/State",
                        "@type": "hydra:Link",
                        "description": "Class for drone state objects",
                        "domain": "http://localhost:8080/api/vocab?resource=EntryPoint",
                        "label": "State",
                        "range": "http://localhost:8080/api/vocab?resource=State",
                        "supportedOperation": [
                            {
                                "@id": "getstate",
                                "@type": "http://schema.org/FindAction",
                                "description": "null",
                                "expects": "null",
                                "expectsHeader": [],
                                "label": "GetState",
                                "method": "GET",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "State not found",
                                        "statusCode": 404,
                                        "title": ""
                                    },
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "State Returned",
                                        "statusCode": 200,
                                        "title": ""
                                    }
                                ],
                                "returns": "http://localhost:8080/api/vocab?resource=State",
                                "returnsHeader": []
                            }
                        ]
                    },
                    "readable": "true",
                    "required": "null",
                    "writeable": "false"
                },
                {
                    "hydra:description": "The Datastream Class",
                    "hydra:title": "datastream",
                    "property": {
                        "@id": "http://localhost:8080/api/vocab?resource=EntryPoint/Datastream",
                        "@type": "hydra:Link",
                        "description": "Class for a datastream entry",
                        "domain": "http://localhost:8080/api/vocab?resource=EntryPoint",
                        "label": "Datastream",
                        "range": "http://localhost:8080/api/vocab?resource=Datastream",
                        "supportedOperation": [
                            {
                                "@id": "readdatastream",
                                "@type": "http://schema.org/FindAction",
                                "description": "null",
                                "expects": "null",
                                "expectsHeader": [],
                                "label": "ReadDatastream",
                                "method": "GET",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "Data not found",
                                        "statusCode": 404,
                                        "title": ""
                                    },
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "Data returned",
                                        "statusCode": 200,
                                        "title": ""
                                    }
                                ],
                                "returns": "http://localhost:8080/api/vocab?resource=Datastream",
                                "returnsHeader": []
                            },
                            {
                                "@id": "updatedatastream",
                                "@type": "http://schema.org/UpdateAction",
                                "description": "null",
                                "expects": "http://localhost:8080/api/vocab?resource=Datastream",
                                "expectsHeader": [],
                                "label": "UpdateDatastream",
                                "method": "POST",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "Data updated",
                                        "statusCode": 200,
                                        "title": ""
                                    }
                                ],
                                "returns": "null",
                                "returnsHeader": []
                            },
                            {
                                "@id": "deletedatastream",
                                "@type": "http://schema.org/DeleteAction",
                                "description": "null",
                                "expects": "null",
                                "expectsHeader": [],
                                "label": "DeleteDatastream",
                                "method": "DELETE",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "Data deleted",
                                        "statusCode": 200,
                                        "title": ""
                                    }
                                ],
                                "returns": "null",
                                "returnsHeader": []
                            }
                        ]
                    },
                    "readable": "true",
                    "required": "null",
                    "writeable": "false"
                },
                {
                    "hydra:description": "The LogEntry Class",
                    "hydra:title": "logentry",
                    "property": {
                        "@id": "http://localhost:8080/api/vocab?resource=EntryPoint/LogEntry",
                        "@type": "hydra:Link",
                        "description": "Class for a log entry",
                        "domain": "http://localhost:8080/api/vocab?resource=EntryPoint",
                        "label": "LogEntry",
                        "range": "http://localhost:8080/api/vocab?resource=LogEntry",
                        "supportedOperation": [
                            {
                                "@id": "getlog",
                                "@type": "http://schema.org/FindAction",
                                "description": "null",
                                "expects": "null",
                                "expectsHeader": [],
                                "label": "GetLog",
                                "method": "GET",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "Log entry not found",
                                        "statusCode": 404,
                                        "title": ""
                                    },
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "Log entry returned",
                                        "statusCode": 200,
                                        "title": ""
                                    }
                                ],
                                "returns": "http://localhost:8080/api/vocab?resource=LogEntry",
                                "returnsHeader": []
                            },
                            {
                                "@id": "addlog",
                                "@type": "http://schema.org/AddAction",
                                "description": "null",
                                "expects": "http://localhost:8080/api/vocab?resource=LogEntry",
                                "expectsHeader": [],
                                "label": "AddLog",
                                "method": "PUT",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "Log entry created",
                                        "statusCode": 201,
                                        "title": ""
                                    }
                                ],
                                "returns": "null",
                                "returnsHeader": []
                            }
                        ]
                    },
                    "readable": "true",
                    "required": "null",
                    "writeable": "false"
                },
                {
                    "hydra:description": "The Area Class",
                    "hydra:title": "area",
                    "property": {
                        "@id": "http://localhost:8080/api/vocab?resource=EntryPoint/Area",
                        "@type": "hydra:Link",
                        "description": "Class for Area of Interest of the server",
                        "domain": "http://localhost:8080/api/vocab?resource=EntryPoint",
                        "label": "Area",
                        "range": "http://localhost:8080/api/vocab?resource=Area",
                        "supportedOperation": [
                            {
                                "@id": "updatearea",
                                "@type": "http://schema.org/UpdateAction",
                                "description": "null",
                                "expects": "http://localhost:8080/api/vocab?resource=Area",
                                "expectsHeader": [],
                                "label": "UpdateArea",
                                "method": "POST",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "Area of interest changed",
                                        "statusCode": 200,
                                        "title": ""
                                    }
                                ],
                                "returns": "null",
                                "returnsHeader": []
                            },
                            {
                                "@id": "getarea",
                                "@type": "http://schema.org/FindAction",
                                "description": "null",
                                "expects": "null",
                                "expectsHeader": [],
                                "label": "GetArea",
                                "method": "GET",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "Area of interest not found",
                                        "statusCode": 200,
                                        "title": ""
                                    },
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "Area of interest returned",
                                        "statusCode": 200,
                                        "title": ""
                                    }
                                ],
                                "returns": "http://localhost:8080/api/vocab?resource=Area",
                                "returnsHeader": []
                            }
                        ]
                    },
                    "readable": "true",
                    "required": "null",
                    "writeable": "false"
                },
                {
                    "hydra:description": "The Command Class",
                    "hydra:title": "command",
                    "property": {
                        "@id": "http://localhost:8080/api/vocab?resource=EntryPoint/Command",
                        "@type": "hydra:Link",
                        "description": "Class for drone commands",
                        "domain": "http://localhost:8080/api/vocab?resource=EntryPoint",
                        "label": "Command",
                        "range": "http://localhost:8080/api/vocab?resource=Command",
                        "supportedOperation": [
                            {
                                "@id": "getcommand",
                                "@type": "http://schema.org/FindAction",
                                "description": "null",
                                "expects": "null",
                                "expectsHeader": [],
                                "label": "GetCommand",
                                "method": "GET",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "Command not found",
                                        "statusCode": 404,
                                        "title": ""
                                    },
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "Command Returned",
                                        "statusCode": 200,
                                        "title": ""
                                    }
                                ],
                                "returns": "http://localhost:8080/api/vocab?resource=Command",
                                "returnsHeader": []
                            },
                            {
                                "@id": "addcommand",
                                "@type": "http://schema.org/AddAction",
                                "description": "null",
                                "expects": "http://localhost:8080/api/vocab?resource=Command",
                                "expectsHeader": [],
                                "label": "AddCommand",
                                "method": "PUT",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "Command added",
                                        "statusCode": 201,
                                        "title": ""
                                    }
                                ],
                                "returns": "null",
                                "returnsHeader": []
                            },
                            {
                                "@id": "deletecommand",
                                "@type": "http://schema.org/DeleteAction",
                                "description": "null",
                                "expects": "null",
                                "expectsHeader": [],
                                "label": "DeleteCommand",
                                "method": "DELETE",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "Command deleted",
                                        "statusCode": 201,
                                        "title": ""
                                    }
                                ],
                                "returns": "null",
                                "returnsHeader": []
                            }
                        ]
                    },
                    "readable": "true",
                    "required": "null",
                    "writeable": "false"
                },
                {
                    "hydra:description": "The Message Class",
                    "hydra:title": "message",
                    "property": {
                        "@id": "http://localhost:8080/api/vocab?resource=EntryPoint/Message",
                        "@type": "hydra:Link",
                        "description": "Class for messages received by the GUI interface",
                        "domain": "http://localhost:8080/api/vocab?resource=EntryPoint",
                        "label": "Message",
                        "range": "http://localhost:8080/api/vocab?resource=Message",
                        "supportedOperation": [
                            {
                                "@id": "getmessage",
                                "@type": "http://schema.org/FindAction",
                                "description": "null",
                                "expects": "null",
                                "expectsHeader": [],
                                "label": "GetMessage",
                                "method": "GET",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "Message not found",
                                        "statusCode": 200,
                                        "title": ""
                                    },
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "Message returned",
                                        "statusCode": 200,
                                        "title": ""
                                    }
                                ],
                                "returns": "http://localhost:8080/api/vocab?resource=Message",
                                "returnsHeader": []
                            },
                            {
                                "@id": "deletemessage",
                                "@type": "http://schema.org/DeleteAction",
                                "description": "null",
                                "expects": "null",
                                "expectsHeader": [],
                                "label": "DeleteMessage",
                                "method": "DELETE",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "Message deleted",
                                        "statusCode": 200,
                                        "title": ""
                                    }
                                ],
                                "returns": "null",
                                "returnsHeader": []
                            }
                        ]
                    },
                    "readable": "true",
                    "required": "null",
                    "writeable": "false"
                },
                {
                    "hydra:description": "The DroneCollection collection",
                    "hydra:title": "dronecollection",
                    "property": {
                        "@id": "http://localhost:8080/api/vocab?resource=EntryPoint/DroneCollection",
                        "@type": "hydra:Link",
                        "description": "The DroneCollection collection",
                        "domain": "http://localhost:8080/api/vocab?resource=EntryPoint",
                        "label": "DroneCollection",
                        "manages": {
                            "object": "http://localhost:8080/api/vocab?resource=Drone",
                            "property": "rdfs:type"
                        },
                        "range": "http://localhost:8080/api/vocab?resource=DroneCollection",
                        "supportedOperation": [
                            {
                                "@id": "_:dronecollection_retrieve",
                                "@type": "http://schema.org/FindAction",
                                "description": "Retrieves all the members of DroneCollection",
                                "expects": "null",
                                "expectsHeader": [],
                                "method": "GET",
                                "possibleStatus": [],
                                "returns": "http://localhost:8080/api/vocab?resource=Drone",
                                "returnsHeader": []
                            },
                            {
                                "@id": "_:dronecollection_create",
                                "@type": "http://schema.org/AddAction",
                                "description": "Create new member in DroneCollection",
                                "expects": "http://localhost:8080/api/vocab?resource=Drone",
                                "expectsHeader": [],
                                "method": "PUT",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "A new member in DroneCollection created",
                                        "statusCode": 201,
                                        "title": ""
                                    }
                                ],
                                "returns": "http://localhost:8080/api/vocab?resource=Drone",
                                "returnsHeader": []
                            },
                            {
                                "@id": "_:dronecollection_update",
                                "@type": "http://schema.org/UpdateAction",
                                "description": "Update member of  DroneCollection ",
                                "expects": "http://localhost:8080/api/vocab?resource=Drone",
                                "expectsHeader": [],
                                "method": "POST",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "If the entity was updatedfrom DroneCollection.",
                                        "statusCode": 200,
                                        "title": ""
                                    }
                                ],
                                "returns": "http://localhost:8080/api/vocab?resource=Drone",
                                "returnsHeader": []
                            },
                            {
                                "@id": "_:dronecollection_delete",
                                "@type": "http://schema.org/DeleteAction",
                                "description": "Delete member of DroneCollection ",
                                "expects": "http://localhost:8080/api/vocab?resource=Drone",
                                "expectsHeader": [],
                                "method": "DELETE",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "If entity was deletedsuccessfully from DroneCollection.",
                                        "statusCode": 200,
                                        "title": ""
                                    }
                                ],
                                "returns": "http://localhost:8080/api/vocab?resource=Drone",
                                "returnsHeader": []
                            }
                        ]
                    },
                    "readable": "true",
                    "required": "null",
                    "writeable": "false"
                },
                {
                    "hydra:description": "The StateCollection collection",
                    "hydra:title": "statecollection",
                    "property": {
                        "@id": "http://localhost:8080/api/vocab?resource=EntryPoint/StateCollection",
                        "@type": "hydra:Link",
                        "description": "The StateCollection collection",
                        "domain": "http://localhost:8080/api/vocab?resource=EntryPoint",
                        "label": "StateCollection",
                        "manages": {
                            "object": "http://localhost:8080/api/vocab?resource=State",
                            "property": "rdfs:type"
                        },
                        "range": "http://localhost:8080/api/vocab?resource=StateCollection",
                        "supportedOperation": [
                            {
                                "@id": "_:statecollection_retrieve",
                                "@type": "http://schema.org/FindAction",
                                "description": "Retrieves all the members of StateCollection",
                                "expects": "null",
                                "expectsHeader": [],
                                "method": "GET",
                                "possibleStatus": [],
                                "returns": "http://localhost:8080/api/vocab?resource=State",
                                "returnsHeader": []
                            },
                            {
                                "@id": "_:statecollection_create",
                                "@type": "http://schema.org/AddAction",
                                "description": "Create new member in StateCollection",
                                "expects": "http://localhost:8080/api/vocab?resource=State",
                                "expectsHeader": [],
                                "method": "PUT",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "A new member in StateCollection created",
                                        "statusCode": 201,
                                        "title": ""
                                    }
                                ],
                                "returns": "http://localhost:8080/api/vocab?resource=State",
                                "returnsHeader": []
                            },
                            {
                                "@id": "_:statecollection_update",
                                "@type": "http://schema.org/UpdateAction",
                                "description": "Update member of  StateCollection ",
                                "expects": "http://localhost:8080/api/vocab?resource=State",
                                "expectsHeader": [],
                                "method": "POST",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "If the entity was updatedfrom StateCollection.",
                                        "statusCode": 200,
                                        "title": ""
                                    }
                                ],
                                "returns": "http://localhost:8080/api/vocab?resource=State",
                                "returnsHeader": []
                            },
                            {
                                "@id": "_:statecollection_delete",
                                "@type": "http://schema.org/DeleteAction",
                                "description": "Delete member of StateCollection ",
                                "expects": "http://localhost:8080/api/vocab?resource=State",
                                "expectsHeader": [],
                                "method": "DELETE",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "If entity was deletedsuccessfully from StateCollection.",
                                        "statusCode": 200,
                                        "title": ""
                                    }
                                ],
                                "returns": "http://localhost:8080/api/vocab?resource=State",
                                "returnsHeader": []
                            }
                        ]
                    },
                    "readable": "true",
                    "required": "null",
                    "writeable": "false"
                },
                {
                    "hydra:description": "The DatastreamCollection collection",
                    "hydra:title": "datastreamcollection",
                    "property": {
                        "@id": "http://localhost:8080/api/vocab?resource=EntryPoint/DatastreamCollection",
                        "@type": "hydra:Link",
                        "description": "The DatastreamCollection collection",
                        "domain": "http://localhost:8080/api/vocab?resource=EntryPoint",
                        "label": "DatastreamCollection",
                        "manages": {
                            "object": "http://localhost:8080/api/vocab?resource=Datastream",
                            "property": "rdfs:type"
                        },
                        "range": "http://localhost:8080/api/vocab?resource=DatastreamCollection",
                        "supportedOperation": [
                            {
                                "@id": "_:datastreamcollection_retrieve",
                                "@type": "http://schema.org/FindAction",
                                "description": "Retrieves all the members of DatastreamCollection",
                                "expects": "null",
                                "expectsHeader": [],
                                "method": "GET",
                                "possibleStatus": [],
                                "returns": "http://localhost:8080/api/vocab?resource=Datastream",
                                "returnsHeader": []
                            },
                            {
                                "@id": "_:datastreamcollection_create",
                                "@type": "http://schema.org/AddAction",
                                "description": "Create new member in DatastreamCollection",
                                "expects": "http://localhost:8080/api/vocab?resource=Datastream",
                                "expectsHeader": [],
                                "method": "PUT",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "A new member in DatastreamCollection created",
                                        "statusCode": 201,
                                        "title": ""
                                    }
                                ],
                                "returns": "http://localhost:8080/api/vocab?resource=Datastream",
                                "returnsHeader": []
                            },
                            {
                                "@id": "_:datastreamcollection_update",
                                "@type": "http://schema.org/UpdateAction",
                                "description": "Update member of  DatastreamCollection ",
                                "expects": "http://localhost:8080/api/vocab?resource=Datastream",
                                "expectsHeader": [],
                                "method": "POST",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "If the entity was updatedfrom DatastreamCollection.",
                                        "statusCode": 200,
                                        "title": ""
                                    }
                                ],
                                "returns": "http://localhost:8080/api/vocab?resource=Datastream",
                                "returnsHeader": []
                            },
                            {
                                "@id": "_:datastreamcollection_delete",
                                "@type": "http://schema.org/DeleteAction",
                                "description": "Delete member of DatastreamCollection ",
                                "expects": "http://localhost:8080/api/vocab?resource=Datastream",
                                "expectsHeader": [],
                                "method": "DELETE",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "If entity was deletedsuccessfully from DatastreamCollection.",
                                        "statusCode": 200,
                                        "title": ""
                                    }
                                ],
                                "returns": "http://localhost:8080/api/vocab?resource=Datastream",
                                "returnsHeader": []
                            }
                        ]
                    },
                    "readable": "true",
                    "required": "null",
                    "writeable": "false"
                },
                {
                    "hydra:description": "The LogEntryCollection collection",
                    "hydra:title": "logentrycollection",
                    "property": {
                        "@id": "http://localhost:8080/api/vocab?resource=EntryPoint/LogEntryCollection",
                        "@type": "hydra:Link",
                        "description": "The LogEntryCollection collection",
                        "domain": "http://localhost:8080/api/vocab?resource=EntryPoint",
                        "label": "LogEntryCollection",
                        "manages": {
                            "object": "http://localhost:8080/api/vocab?resource=LogEntry",
                            "property": "rdfs:type"
                        },
                        "range": "http://localhost:8080/api/vocab?resource=LogEntryCollection",
                        "supportedOperation": [
                            {
                                "@id": "_:logentrycollection_retrieve",
                                "@type": "http://schema.org/FindAction",
                                "description": "Retrieves all the members of LogEntryCollection",
                                "expects": "null",
                                "expectsHeader": [],
                                "method": "GET",
                                "possibleStatus": [],
                                "returns": "http://localhost:8080/api/vocab?resource=LogEntry",
                                "returnsHeader": []
                            },
                            {
                                "@id": "_:logentrycollection_create",
                                "@type": "http://schema.org/AddAction",
                                "description": "Create new member in LogEntryCollection",
                                "expects": "http://localhost:8080/api/vocab?resource=LogEntry",
                                "expectsHeader": [],
                                "method": "PUT",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "A new member in LogEntryCollection created",
                                        "statusCode": 201,
                                        "title": ""
                                    }
                                ],
                                "returns": "http://localhost:8080/api/vocab?resource=LogEntry",
                                "returnsHeader": []
                            },
                            {
                                "@id": "_:logentrycollection_update",
                                "@type": "http://schema.org/UpdateAction",
                                "description": "Update member of  LogEntryCollection ",
                                "expects": "http://localhost:8080/api/vocab?resource=LogEntry",
                                "expectsHeader": [],
                                "method": "POST",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "If the entity was updatedfrom LogEntryCollection.",
                                        "statusCode": 200,
                                        "title": ""
                                    }
                                ],
                                "returns": "http://localhost:8080/api/vocab?resource=LogEntry",
                                "returnsHeader": []
                            },
                            {
                                "@id": "_:logentrycollection_delete",
                                "@type": "http://schema.org/DeleteAction",
                                "description": "Delete member of LogEntryCollection ",
                                "expects": "http://localhost:8080/api/vocab?resource=LogEntry",
                                "expectsHeader": [],
                                "method": "DELETE",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "If entity was deletedsuccessfully from LogEntryCollection.",
                                        "statusCode": 200,
                                        "title": ""
                                    }
                                ],
                                "returns": "http://localhost:8080/api/vocab?resource=LogEntry",
                                "returnsHeader": []
                            }
                        ]
                    },
                    "readable": "true",
                    "required": "null",
                    "writeable": "false"
                },
                {
                    "hydra:description": "The CommandCollection collection",
                    "hydra:title": "commandcollection",
                    "property": {
                        "@id": "http://localhost:8080/api/vocab?resource=EntryPoint/CommandCollection",
                        "@type": "hydra:Link",
                        "description": "The CommandCollection collection",
                        "domain": "http://localhost:8080/api/vocab?resource=EntryPoint",
                        "label": "CommandCollection",
                        "manages": {
                            "object": "http://localhost:8080/api/vocab?resource=Command",
                            "property": "rdfs:type"
                        },
                        "range": "http://localhost:8080/api/vocab?resource=CommandCollection",
                        "supportedOperation": [
                            {
                                "@id": "_:commandcollection_retrieve",
                                "@type": "http://schema.org/FindAction",
                                "description": "Retrieves all the members of CommandCollection",
                                "expects": "null",
                                "expectsHeader": [],
                                "method": "GET",
                                "possibleStatus": [],
                                "returns": "http://localhost:8080/api/vocab?resource=Command",
                                "returnsHeader": []
                            },
                            {
                                "@id": "_:commandcollection_create",
                                "@type": "http://schema.org/AddAction",
                                "description": "Create new member in CommandCollection",
                                "expects": "http://localhost:8080/api/vocab?resource=Command",
                                "expectsHeader": [],
                                "method": "PUT",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "A new member in CommandCollection created",
                                        "statusCode": 201,
                                        "title": ""
                                    }
                                ],
                                "returns": "http://localhost:8080/api/vocab?resource=Command",
                                "returnsHeader": []
                            },
                            {
                                "@id": "_:commandcollection_update",
                                "@type": "http://schema.org/UpdateAction",
                                "description": "Update member of  CommandCollection ",
                                "expects": "http://localhost:8080/api/vocab?resource=Command",
                                "expectsHeader": [],
                                "method": "POST",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "If the entity was updatedfrom CommandCollection.",
                                        "statusCode": 200,
                                        "title": ""
                                    }
                                ],
                                "returns": "http://localhost:8080/api/vocab?resource=Command",
                                "returnsHeader": []
                            },
                            {
                                "@id": "_:commandcollection_delete",
                                "@type": "http://schema.org/DeleteAction",
                                "description": "Delete member of CommandCollection ",
                                "expects": "http://localhost:8080/api/vocab?resource=Command",
                                "expectsHeader": [],
                                "method": "DELETE",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "If entity was deletedsuccessfully from CommandCollection.",
                                        "statusCode": 200,
                                        "title": ""
                                    }
                                ],
                                "returns": "http://localhost:8080/api/vocab?resource=Command",
                                "returnsHeader": []
                            }
                        ]
                    },
                    "readable": "true",
                    "required": "null",
                    "writeable": "false"
                },
                {
                    "hydra:description": "The MessageCollection collection",
                    "hydra:title": "messagecollection",
                    "property": {
                        "@id": "http://localhost:8080/api/vocab?resource=EntryPoint/MessageCollection",
                        "@type": "hydra:Link",
                        "description": "The MessageCollection collection",
                        "domain": "http://localhost:8080/api/vocab?resource=EntryPoint",
                        "label": "MessageCollection",
                        "manages": {
                            "object": "http://localhost:8080/api/vocab?resource=Message",
                            "property": "rdfs:type"
                        },
                        "range": "http://localhost:8080/api/vocab?resource=MessageCollection",
                        "supportedOperation": [
                            {
                                "@id": "_:messagecollection_retrieve",
                                "@type": "http://schema.org/FindAction",
                                "description": "Retrieves all the members of MessageCollection",
                                "expects": "null",
                                "expectsHeader": [],
                                "method": "GET",
                                "possibleStatus": [],
                                "returns": "http://localhost:8080/api/vocab?resource=Message",
                                "returnsHeader": []
                            },
                            {
                                "@id": "_:messagecollection_create",
                                "@type": "http://schema.org/AddAction",
                                "description": "Create new member in MessageCollection",
                                "expects": "http://localhost:8080/api/vocab?resource=Message",
                                "expectsHeader": [],
                                "method": "PUT",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "A new member in MessageCollection created",
                                        "statusCode": 201,
                                        "title": ""
                                    }
                                ],
                                "returns": "http://localhost:8080/api/vocab?resource=Message",
                                "returnsHeader": []
                            },
                            {
                                "@id": "_:messagecollection_update",
                                "@type": "http://schema.org/UpdateAction",
                                "description": "Update member of  MessageCollection ",
                                "expects": "http://localhost:8080/api/vocab?resource=Message",
                                "expectsHeader": [],
                                "method": "POST",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "If the entity was updatedfrom MessageCollection.",
                                        "statusCode": 200,
                                        "title": ""
                                    }
                                ],
                                "returns": "http://localhost:8080/api/vocab?resource=Message",
                                "returnsHeader": []
                            },
                            {
                                "@id": "_:messagecollection_delete",
                                "@type": "http://schema.org/DeleteAction",
                                "description": "Delete member of MessageCollection ",
                                "expects": "http://localhost:8080/api/vocab?resource=Message",
                                "expectsHeader": [],
                                "method": "DELETE",
                                "possibleStatus": [
                                    {
                                        "@context": "https://raw.githubusercontent.com/HydraCG/Specifications/master/spec/latest/core/core.jsonld",
                                        "@type": "Status",
                                        "description": "If entity was deletedsuccessfully from MessageCollection.",
                                        "statusCode": 200,
                                        "title": ""
                                    }
                                ],
                                "returns": "http://localhost:8080/api/vocab?resource=Message",
                                "returnsHeader": []
                            }
                        ]
                    },
                    "readable": "true",
                    "required": "null",
                    "writeable": "false"
                }
            ],
            "title": "EntryPoint"
        }
    ],
    "title": "API Doc for the server side API"
}


from flask import Flask, redirect, request
import uuid
from flask_cors import CORS
from flask_restful import Api
from hydrus.resources import (Index, Vocab, Contexts, Entrypoint,
                              ItemCollection, Item, Items, ItemMember)


def app_factory(api_name: str = "api", vocab_route: str = "vocab") -> Flask:
    """
    Create an app object
    :param api_name : Name of the api
    :param vocab_route : The route at which the vocab of the apidoc is present
    :return : API with all routes directed at /[api_name].
    """

    app = Flask(__name__)
    app.config['SECRET_KEY'] = 'secret key'
    CORS(app)
    app.url_map.strict_slashes = False
    api = Api(app)

    # Redirecting root_path to root_path/api_name
    if api_name:
        @app.route("/")
        def root_url():
            return redirect(f"/{api_name}/")

    api.add_resource(Index, f"/{api_name}/", endpoint="api")
    api.add_resource(Vocab, f"/{api_name}/{vocab_route}", endpoint="vocab")
    api.add_resource(
        Contexts,
        f"/{api_name}/contexts/<string:category>.jsonld",
        endpoint="contexts")
    api.add_resource(
        Entrypoint,
        f"/{api_name}/contexts/EntryPoint.jsonld",
        endpoint="main_entrypoint")
    api.add_resource(
        ItemCollection,
        f"/{api_name}/<string:path>",
        endpoint="item_collection")
    api.add_resource(
        Item,
        f"/{api_name}/<string:path>/<uuid:id_>",
        endpoint="item")
    api.add_resource(
        ItemMember,
        f"/{api_name}/<string:path>/<uuid:collection_id_>/<uuid:id_>",
        endpoint="member")
    api.add_resource(
        Items,
        f"/{api_name}/<string:path>/add/<int_list>",
        f"/{api_name}/<string:path>/add",
        f"/{api_name}/<string:path>/delete/<int_list>")

    return app



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['hydrus',
 'hydrus.data',
 'hydrus.data.migrations',
 'hydrus.samples',
 'hydrus.tests',
 'hydrus.tests.functional',
 'hydrus.tests.unit']

package_data = \
{'': ['*']}

install_requires = \
['Flask-Cors>=3.0.10',
 'Flask-RESTful>=0.3.9',
 'Flask-SocketIO>=4.2.1,<4.3.0',
 'Flask>=2.0.0',
 'Jinja2>=3.0',
 'Mako>=1.1.0,<1.2.0',
 'MarkupSafe>=2.0.0rc2',
 'PyYAML>=5.4',
 'SQLAlchemy>=1.3.12,<1.4.0',
 'Werkzeug>=2.0',
 'alembic>=1.0.10,<1.1.0',
 'aniso8601>=3.0.2,<3.1.0',
 'appdirs>=1.4.3,<1.5.0',
 'blinker>=1.4,<1.5',
 'certifi>=2019.11.28,<2019.12.0',
 'click>=7.1.2,<7.2.0',
 'flake8>=3.8.3,<3.9.0',
 'gevent-websocket>=0.10.1,<0.11.0',
 'gevent>=20.12.0,<20.13.0',
 'greenlet>=0.4.17,<0.5.0',
 'hydra_openapi_parser>=0.2',
 'hydra_python_core>=0.3',
 'itsdangerous>=2.0.0',
 'lifter>=0.4.1,<0.5.0',
 'packaging>=20.9,<20.10',
 'pep8>=1.7.1,<1.8.0',
 'persisting-theory>=0.2.1,<0.3.0',
 'psycopg2-binary>=2.8.6,<2.9.0',
 'pyparsing>=2.4.6,<2.5.0',
 'pytest>=5.4.1,<5.5.0',
 'python-dateutil>=2.8.1,<2.9.0',
 'python-editor>=1.0.4,<1.1.0',
 'python-engineio>=3.11.2,<3.12.0',
 'python-socketio>=4.4.0,<4.5.0',
 'pytz>=2019.3,<2019.4',
 'six>=1.13.0,<1.14.0',
 'structlog>=21.1.0',
 'thespian>=3.9.11,<3.10.0']

setup_kwargs = {
    'name': 'hydrus',
    'version': '0.4.3',
    'description': 'Hydra Ecosystem Flagship Server. Deploy REST data for Web 3.0',
    'long_description': None,
    'author': 'Hydra Ecosystem',
    'author_email': None,
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6.8',
}


setup(**setup_kwargs)



"""Helper functions for items"""
import json

from flask import jsonify, request
from hydra_python_core.doc_writer import HydraStatus, HydraError

from hydrus.data import crud
from hydrus.data.exceptions import (
    ClassNotFound,
    InstanceNotFound,
    InstanceExists,
    PropertyNotFound,
    MemberInstanceNotFound)
from hydrus.helpers import (
    set_response_headers,
    finalize_response,
    hydrafy,
    getType,
    send_sync_update,
    get_link_props,
    error_response,
    validate_object,
    parse_collection_members,
)
from hydrus.utils import (
    get_session,
    get_api_name,
    get_hydrus_server_url,
    get_collections_and_parsed_classes)
from hydrus.socketio_factory import socketio


def items_get_check_support(id_, class_type, class_path, path, is_collection=False):
    """Check if class_type supports GET operation"""
    try:
        # Try getting the Item based on ID and Class type
        response = crud.get(
            id_,
            class_type,
            api_name=get_api_name(),
            session=get_session(),
            path=path,
            collection=is_collection)

        response = finalize_response(class_path, response)
        return set_response_headers(
            jsonify(hydrafy(response, path=path)))

    except (ClassNotFound, InstanceNotFound) as e:
        error = e.get_HTTP()
        return error_response(error)


def items_post_check_support(id_, object_, class_path, path, is_collection):
    """Check if class_type supports POST operation"""
    collections, parsed_classes = get_collections_and_parsed_classes()
    if path in parsed_classes:
        class_path = path
        obj_type = getType(path, "PUT")
    elif path in collections:
        collection = collections[path]["collection"]
        class_path = collection.path
        obj_type = collection.name
    link_props, link_type_check = get_link_props(class_path, object_)
    # Load new object and type
    if (validate_object(object_, obj_type, class_path) and link_type_check):
        if is_collection:
            object_ = parse_collection_members(object_)
        try:
            # Update the right ID if the object is valid and matches
            # type of Item
            object_id = crud.update(
                object_=object_,
                id_=id_,
                type_=object_["@type"],
                session=get_session(),
                api_name=get_api_name(),
                collection=is_collection)
            method = "POST"
            resource_url = f"{get_hydrus_server_url()}{get_api_name()}/{path}/{object_id}"
            last_job_id = crud.get_last_modification_job_id(
                session=get_session())
            new_job_id = crud.insert_modification_record(method, resource_url,
                                                         session=get_session())
            send_sync_update(socketio=socketio, new_job_id=new_job_id,
                             last_job_id=last_job_id, method=method,
                             resource_url=resource_url)
            headers_ = [{"Location": resource_url}]
            status_description = (f"Object with ID {object_id} successfully "
                                  "updated")
            status = HydraStatus(
                code=200, title="Object updated", desc=status_description)
            status_response = status.generate()
            status_response["iri"] = resource_url
            return set_response_headers(jsonify(status_response),
                                        headers=headers_)

        except (ClassNotFound,
                InstanceNotFound,
                InstanceExists,
                PropertyNotFound
                ) as e:
            error = e.get_HTTP()
            return error_response(error)
    else:
        error = HydraError(code=400, title="Data is not valid")
        return error_response(error)


def items_put_check_support(id_, class_path, path, is_collection):
    """Check if class_type supports PUT operation"""
    object_ = json.loads(request.data.decode('utf-8'))
    collections, parsed_classes = get_collections_and_parsed_classes()
    if path in parsed_classes:
        class_path = path
        obj_type = getType(path, "PUT")
    elif path in collections:
        collection = collections[path]["collection"]
        class_path = collection.path
        obj_type = collection.name
    link_props, link_type_check = get_link_props(class_path, object_)
    # Load new object and type
    if (validate_object(object_, obj_type, class_path) and link_type_check):
        if is_collection:
            object_ = parse_collection_members(object_)
        try:
            # Add the object with given ID
            object_id = crud.insert(object_=object_, id_=id_,
                                    session=get_session(), collection=is_collection)
            resource_url = f"{get_hydrus_server_url()}{get_api_name()}/{path}/{object_id}"
            headers_ = [{"Location": resource_url}]
            status_description = f"Object with ID {object_id} successfully added"
            status = HydraStatus(code=201, title="Object successfully added.",
                                 desc=status_description)
            status_response = status.generate()
            status_response["iri"] = resource_url
            return set_response_headers(
                jsonify(status_response), headers=headers_,
                status_code=status.code)
        except (ClassNotFound, InstanceExists, PropertyNotFound) as e:
            error = e.get_HTTP()
            return error_response(error)
    else:
        error = HydraError(code=400, title="Data is not valid")
        return error_response(error)


def items_delete_check_support(id_, class_type, path, is_collection):
    """Check if class_type supports PUT operation"""
    try:
        # Delete the Item with ID == id_
        # for colletions, id_ is corresponding to their collection_id and not the id_
        # primary key
        crud.delete(id_, class_type, session=get_session(), collection=is_collection)
        method = "DELETE"
        resource_url = f"{get_hydrus_server_url()}{get_api_name()}/{path}/{id_}"
        last_job_id = crud.get_last_modification_job_id(session=get_session())
        new_job_id = crud.insert_modification_record(method, resource_url,
                                                     session=get_session())
        send_sync_update(socketio=socketio, new_job_id=new_job_id,
                         last_job_id=last_job_id, method=method,
                         resource_url=resource_url)
        status_description = f"Object with ID {id_} successfully deleted"
        status = HydraStatus(code=200, title="Object successfully deleted.",
                             desc=status_description)
        return set_response_headers(jsonify(status.generate()))

    except (ClassNotFound, InstanceNotFound) as e:
        error = e.get_HTTP()
        return error_response(error)


def member_get_check_support(collection_id, member_id, class_type, class_path, path):
    """Check if class_type supports GET operation"""
    try:
        # Try getting the Item based on Collection ID and Member ID and Class type
        response = crud.get_member(
            collection_id,
            member_id,
            class_type,
            api_name=get_api_name(),
            session=get_session(),
            path=path)

        response = finalize_response(class_path, response)
        return set_response_headers(
            jsonify(hydrafy(response, path=path)))

    except (ClassNotFound, MemberInstanceNotFound) as e:
        error = e.get_HTTP()
        return error_response(error)


def member_delete_check_support(collection_id, member_id, class_type, path):
    """Check if class_type supports DELETE operation"""
    try:
        # Delete the Item with IDs collection_id and member_id
        # collection_id is id of a collection
        # member_id is the id of member of a collection
        crud.delete_member(
            collection_id,
            member_id,
            class_type,
            session=get_session()
        )

        method = "DELETE"
        resource_url = f"{get_hydrus_server_url()}{get_api_name()}/{path}/{collection_id}"
        last_job_id = crud.get_last_modification_job_id(session=get_session())
        new_job_id = crud.insert_modification_record(method, resource_url,
                                                     session=get_session())
        status_description = (f"Object with ID {member_id} successfully"
                              f" deleted from Collection with ID {collection_id}")
        status = HydraStatus(code=200, title="Object successfully deleted.",
                             desc=status_description)
        return set_response_headers(jsonify(status.generate()))

    except (ClassNotFound, MemberInstanceNotFound) as e:
        error = e.get_HTTP()
        return error_response(error)



"""
Global variables are loaded or set here:
    DEBUG
    PORT
    API_NAME
    DB_URL
    APIDOC_OBJ
    HYDRUS_SERVER_URL
    FOUND_DOC
"""
import os
import json
import yaml
import logging
from os.path import abspath, dirname
from pathlib import Path
from importlib.machinery import SourceFileLoader
from hydra_openapi_parser.openapi_parser import parse

logger = logging.getLogger(__file__)

try:
    DEBUG = bool(os.environ['DEBUG'])
except KeyError:
    DEBUG = False

# load form environment (as many globals as possible shall be in
# environment configuration)

PORT = int(os.environ['PORT']) if 'PORT' in dict(os.environ).keys() else 8080
API_NAME = os.environ['API_NAME'] if 'API_NAME' in dict(os.environ).keys() else 'api'
DB_URL = os.environ['DB_URL'] if 'DB_URL' in dict(os.environ).keys() else 'sqlite:///database.db'


def get_apidoc_path():
    """
    Get the path of the apidoc.

    :return - Tuple (path, boolean). path denotes path of the apidoc.
    If apidoc is not present at specified path then it falls back at sample apidoc.
    boolean is true if the apidoc is present at the specified path.
    boolean is false if sample apidoc is being used.
    """
    cwd_path = Path(dirname(dirname(abspath(__file__))))
    try:
        apidoc_env = os.environ['APIDOC_REL_PATH']
        apidoc_path = cwd_path / Path(apidoc_env)
        found_doc = True
    except KeyError:
        found_doc = False
        apidoc_path = cwd_path / 'hydrus' / 'samples' / 'hydra_doc_sample.py'
    return (apidoc_path, found_doc)


def load_apidoc(path):
    """
    Parses docs of .jsonld, .py, .yaml format and loads apidoc from the given path.

    :param path - Path for the apidoc to be loaded
    :return - apidoc
    :Raises:
        FileNotFoundError: If the wrong path of hydradoc is specified.
        BaseException: If hydradoc is specified in wrong format.
    """
    path = str(path)
    try:
        apidoc_format = path.split(".")[-1]
        if apidoc_format == 'jsonld':
            with open(path, 'r') as f:
                api_doc = json.load(f)
        elif apidoc_format == 'py':
            api_doc = SourceFileLoader(
                'doc', path).load_module().doc
        elif apidoc_format == 'yaml':
            with open(path, 'r') as stream:
                api_doc = parse(yaml.load(stream))
        else:
            raise(
                "Error - hydradoc format not supported."
                "The supported formats are .py, .jsonld and .yaml")

        logger.info(f'APIDOC path loaded from: {path}')
        return api_doc
    except FileNotFoundError:
        logger.critical(f'No Hydra ApiDoc file to load has been found'
                        f' at {path}. Cannot set APIDOC_OBJ')
        raise
    except BaseException:
        logger.critical("Problem parsing specified hydradoc file")
        raise


def get_host_domain():
    """
    Returns host domain.
    """
    HOST_DOMAIN = f'http://localhost:{PORT}'
    return HOST_DOMAIN


(path, FOUND_DOC) = get_apidoc_path()
APIDOC_OBJ = load_apidoc(path)
HYDRUS_SERVER_URL = f'http://localhost:{PORT}/'



"""Sample to create Hydra APIDocumentation using doc_writer."""

from hydra_python_core.doc_writer import (HydraDoc, HydraClass, HydraClassProp, HydraClassOp,
                                            HydraStatus, HydraLink,HydraCollection)
from typing import Any, Dict, Union

# Creating the HydraDoc object, this is the primary class for the Doc
API_NAME = "api"                # Name of the API, will serve as EntryPoint
BASE_URL = "http://hydrus.com/"    # The base url at which the API is hosted
# NOTE: The API will be accessible at BASE_URL + ENTRY_POINT
# (http://hydrus.com/api)
# Create ApiDoc Object
api_doc = HydraDoc(API_NAME,
                   "Title for the API Documentation",
                   "Description for the API Documentation",
                   API_NAME,
                   BASE_URL,
                   "vocab")


# Creating classes for the API
class_title = "dummyClass"                      # Title of the Class
class_description = "A dummyClass for demo"     # Description of the class
class_ = HydraClass(class_title, class_description, endpoint=False)


# Class with single instance

class_2_title = "singleClass"
class_2_description = "A non collection class"
class_2 = HydraClass(class_2_title,
                     class_2_description, endpoint=True)

# Another class with single instance, will be used as nested class

class_1_title = "anotherSingleClass"
class_1_description = "An another non collection class"
class_1 = HydraClass(class_1_title,
                     class_1_description, endpoint=True)

# Class not having any methods except put and get
class_3_title = "extraClass"
class_3_description = "Class without any explicit methods"
class_3 = HydraClass(class_3_title,
                     class_3_description, endpoint=False)

collection_name = "Extraclasses"
collection_title = "ExtraClass collection"
collection_description = "This collection comprises of instances of ExtraClass"
# add explicit statements about members of the collection
# Following manages block means every member of this collection is of type class_3
collection_managed_by = {
    "property": "rdf:type",
    "object": class_3.id_,
}
collection_1 = HydraCollection(collection_name=collection_name,
                               collection_description=collection_description, manages=collection_managed_by, get=True,
                               post=True, collection_path="EcTest")

collection2_title = "dummyClass collection"
collection2_name = "dummyclasses"
collection2_description = "This collection comprises of instances of dummyClass"
collection2_managed_by = {
    "property": "rdf:type",
    "object": class_.id_,
}

collection_2 = HydraCollection(collection_name=collection2_name,
                               collection_description=collection2_description, manages=collection2_managed_by, get=True,
                               post=True, collection_path="DcTest")

# NOTE: Setting endpoint=True creates an endpoint for the class itself, this is usually for classes
#       that have single instances.
#       These classes should not ideally have a Collection, although
#       Hydrus will allow creation of Collections for them


# Create new properties for the class
# The URI of the class of the property
prop1_uri = "http://props.hydrus.com/prop1"
prop1_title = "Prop1"                   # Title of the property

dummyProp1 = HydraClassProp(prop1_uri, prop1_title,
                            required=False, read=False, write=True)


prop2_uri = "http://props.hydrus.com/prop2"
prop2_title = "Prop2"

dummyProp2 = HydraClassProp(prop2_uri, prop2_title,
                            required=False, read=False, write=True)
# NOTE: Properties that are required=True must be added during class object creation
#       Properties that are read=True are read only
#       Properties that are write=True are writable


# Create operations for the class
op_name = "UpdateClass"  # The name of the operation
op_method = "POST"  # The method of the Operation [GET, POST, PUT, DELETE]
# URI of the object that is expected for the operation
op_expects = class_.id_
op_returns = None   # URI of the object that is returned by the operation
op_returns_header = ["Content-Type", "Content-Length"]
op_expects_header = []
# List of statusCode for the operation
op_status = [HydraStatus(code=200, desc="dummyClass updated.")]

op1 = HydraClassOp(op_name,
                   op_method,
                   op_expects,
                   op_returns,
                   op_expects_header,
                   op_returns_header,
                   op_status)

# Same way add DELETE, PUT and GET operations
op2_status = [HydraStatus(code=200, desc="dummyClass deleted.")]
op2 = HydraClassOp("DeleteClass", "DELETE", None, None, [], [], op2_status)
op3_status = [HydraStatus(code=201, desc="dummyClass successfully added.")]
op3 = HydraClassOp("AddClass", "PUT", class_.id_, None, [], [], op3_status)
op4_status = [HydraStatus(code=200, desc="dummyClass returned.")]
op4 = HydraClassOp("GetClass", "GET", None, class_.id_, [], [], op4_status)

# Operations for non collection class
class_2_op1_status = [HydraStatus(code=200, desc="singleClass changed.")]
class_2_op1 = HydraClassOp("UpdateClass", "POST",
                           class_2.id_, None, [], [], class_2_op1_status)
class_2_op2_status = [HydraStatus(code=200, desc="singleClass deleted.")]
class_2_op2 = HydraClassOp("DeleteClass", "DELETE",
                           None, None, [], [], class_2_op2_status)
class_2_op3_status = [HydraStatus(code=201, desc="singleClass successfully added.")]
class_2_op3 = HydraClassOp(
    "AddClass", "PUT", class_2.id_, None, [], [], class_2_op3_status)
class_2_op4_status = [HydraStatus(code=200, desc="singleClass returned.")]
class_2_op4 = HydraClassOp("GetClass", "GET", None,
                           class_2.id_, [], [], class_2_op4_status)

class_1_op1_status = [HydraStatus(code=200, desc="anotherSingleClass returned.")]
class_1_op1 = HydraClassOp("GetClass", "GET", None,
                           class_1.id_, [], [], class_1_op1_status)
# Add the properties to the classes
class_.add_supported_prop(dummyProp1)
class_.add_supported_prop(dummyProp2)
class_2.add_supported_prop(dummyProp1)
class_2.add_supported_prop(dummyProp2)
dummy_prop_link = HydraLink("singleClass/dummyProp", "dummyProp", domain=class_2.id_,
                            range_=class_.id_)
class_2.add_supported_prop(HydraClassProp(
    dummy_prop_link, "dummyProp", required=False, read=False, write=True))
class_2.add_supported_prop(HydraClassProp(
    class_1.id_, "singleClassProp", required=False, read=False, write=True))
class_1.add_supported_prop(dummyProp1)
# Add the operations to the classes
class_.add_supported_op(op1)
class_.add_supported_op(op2)
class_.add_supported_op(op3)
class_.add_supported_op(op4)
class_2.add_supported_op(class_2_op1)
class_2.add_supported_op(class_2_op2)
class_2.add_supported_op(class_2_op3)
class_2.add_supported_op(class_2_op4)
class_1.add_supported_op(class_1_op1)

# Add the classes to the HydraDoc

api_doc.add_supported_class(class_)
api_doc.add_supported_class(class_3)
api_doc.add_supported_class(class_2)
api_doc.add_supported_class(class_1)
api_doc.add_supported_class(class_1)

# add the collection to the HydraDoc.
api_doc.add_supported_collection(collection_1)
api_doc.add_supported_collection(collection_2)

# Other operations needed for the Doc
# Creates the base Resource Class and adds it to the API Documentation
api_doc.add_baseResource()
# Creates the base Collection Class and adds it to the API Documentation
api_doc.add_baseCollection()
# Generates the EntryPoint object for the Doc using the Classes and Collections
api_doc.gen_EntryPoint()


# Generate the complete API Documentation
doc = api_doc.generate(
)  # type: Union[Dict[str, Any], str]       # Returns the entire API Documentation as a Python dict

if __name__ == "__main__":
    """Print the complete sample Doc in doc_writer_sample_output.py."""
    import json

    dump = json.dumps(doc, indent=4, sort_keys=True)
    doc = '''"""Generated API Documentation sample using doc_writer_sample.py."""
    \ndoc = {}\n'''.format(dump)
    # Python does not recognise null, true and false in JSON format, convert
    # them to string
    doc = doc.replace('true', '"true"')
    doc = doc.replace('false', '"false"')
    doc = doc.replace('null', '"null"')
    with open("samples/doc_writer_sample_output.py", "w") as f:
        f.write(doc)



"""Parser for Hydra APIDocumentation creates Classes and Properties."""

from typing import Any, Dict, List, Set, Optional
from hydra_python_core.doc_writer import HydraDoc


def get_classes(apidoc: HydraDoc) -> List[Dict[str, Any]]:
    """Get all the classes in the APIDocumentation."""
    COLLECTION_ID = "http://www.w3.org/ns/hydra/core#Collection"
    RESOURCE_ID = "http://www.w3.org/ns/hydra/core#Resource"
    ENTRYPOINT_ID = apidoc.entrypoint.entrypoint.id_
    classes = []
    for class_ in apidoc.generate()["supportedClass"]:
        if class_["@id"] not in [COLLECTION_ID, RESOURCE_ID, ENTRYPOINT_ID]:
            classes.append(class_)
    # print(classes)
    return classes


def get_all_properties(classes: List[Dict[str, Any]]) -> Set[str]:
    """Get all the properties in the APIDocumentation."""
    # properties = list()
    prop_names = set()  # type: Set[str]
    for class_ in classes:
        for prop in class_["supportedProperty"]:
            if prop["title"] not in prop_names:
                prop_names.add(prop["title"])
                # properties.append(prop)
    return set(prop_names)

# if __name__ == "__main__":
#     Session = sessionmaker(bind=engine)
#     session = Session()
#
#     doc = doc_gen("test", "test")
#     # Extract all classes with supportedProperty from both
#     classes = get_classes(doc.generate())
#
#     # Extract all properties from both
#     # import pdb; pdb.set_trace()
#     properties = get_all_properties(classes)
#     # Add all the classes
#     insert_classes(classes, session)
#     print("Classes inserted successfully")
#     # Add all the properties
#     insert_properties(properties, session)
#     print("Properties inserted successfully")
